var MAX_NUMBER_OF_USERS = 60;

var m_USunitFormat = true;
var m_numberOfUsers = 0;

$(document).ready(function () {

    window.onerror = parent.window.onerror;
    $("#cancelButton").click(onCancelClicked);
    $("#termsConditionsButton").click(onTermsAndConditionsClicked);
    $("#createButton").click(onCreateClicked);
    $("#weightHelpButton").click(onWeightHelpClicked);
    
    parent.TgomaMessaging.callPlugin(onNumberOfUsersReturned, null, this, parent.TgomaMessaging.USER_MODULE_NAME, "getNumberOfUsers", [])
    parent.TgomaMessaging.callPlugin(function (p_accountPrefs) {
        m_USunitFormat = p_accountPrefs["unitsAreUSFormat"];
        if (m_USunitFormat) {
            $("#weightLabel").text(parent.Tgoma.LanguageManager.getString("weight_pounds"));
            $("#weightLabel").attr("texttag", 'weight_pounds');
        }
        else {
            $("#weightLabel").text(parent.Tgoma.LanguageManager.getString("weight_kgs"));
            $("#weightLabel").attr("texttag", 'weight_kgs');
        }

    }, null, this, parent.TgomaMessaging.USER_MODULE_NAME, "getFamilyAccountFormatPreferences");
});

function onWeightHelpClicked(evt) {
    console.log("[PlayerDetails.htm] onWeightHelpClicked()");
    parent.showAlert("weight_tip");
}

function onNumberOfUsersReturned(p_numberOfUsers) {
    console.log("onNumberOfUsersReturned:" + p_numberOfUsers);
    m_numberOfUsers = p_numberOfUsers;
}

function onCreateClicked(evt) {
    console.log("onCreateClicked()");

    var name = $("#name");
    var nameTips = $("#nameTips");
    var weight = $("#weight");
    var weightTips = $("#weightTips");

    // Remove any error classes that may exist on the fields and their tips.
    nameTips.removeClass("tip-state-visible");
    weightTips.removeClass("tip-state-visible");

    name.removeClass("input-state-error");
    weight.removeClass("input-state-error");

    // Perform validity check on name and weight field
    var _valid = true;

    if (m_numberOfUsers >= MAX_NUMBER_OF_USERS) {
        _valid = false;
        updateTips(name, nameTips, "user_limit_message");
    }

    _valid = checkLength(name, nameTips, "Name", 1, 10) && _valid;
    _valid = checkLength(weight, weightTips, "weight", 1, 3) && _valid;

    if (m_USunitFormat) {
        _valid = checkRegexp($("#weight"), $("#weightTips"), /^\d+$/, "weight_check_1_pounds") && _valid;
        _valid = checkMaxValue($("#weight"), $("#weightTips"), 330, "weight_check_2_pounds") && _valid;
        _valid = checkMinValue($("#weight"), $("#weightTips"), 10, "weight_check_3_pounds") && _valid;
    } else {
        _valid = checkRegexp($("#weight"), $("#weightTips"), /^\d+$/, "weight_check_1_kgs") && _valid;
        _valid = checkMaxValue($("#weight"), $("#weightTips"), 150, "weight_check_2_kgs") && _valid;
        _valid = checkMinValue($("#weight"), $("#weightTips"), 5, "weight_check_3_kgs") && _valid;
    }

    _valid = checkForEmojis($("#name"), $("#nameTips"), "validate_name_error") && _valid;
    _valid = checkRegexp($("#name"), $("#nameTips"), /^[\u0400-\u04FFA-Za-z][-\u0400-\u04FFA-Za-z0-9]*$/, "validate_name_error") && _valid;

    // If it's alright so far...
    if (_valid) {
        // Perform check on name to see if it already exists on the device
        parent.TgomaMessaging.callPlugin(onLocalNameExistanceCheckReturned, null, this, parent.TgomaMessaging.USER_MODULE_NAME, "checkLocalUserExistsOnDevice", [name.val()]);
    }
}

function onCancelClicked(evt) {
    if (parent.getURLParameter("ftue", location) == "true") {
        parent.loadFrame("ftue", "nav-hidden=true");
    } else {
        parent.loadFrame("players");
    }
}

function onLocalNameExistanceCheckReturned(p_nameExists) {
    console.log("onLocalNameExistanceCheckReturned(" + p_nameExists + ")");
    if (p_nameExists) {
        updateTips($("#name"), $("#nameTips"), "name_in_use");
    }
    else {
        var weight = parseInt($("#weight").val());
        if (m_USunitFormat) {
            weight = parent.Tgoma.Utils.convertPoundsToKilograms(weight);
        }
        parent.TgomaMessaging.callPlugin(onUserAccountCreated, null, this, parent.TgomaMessaging.USER_MODULE_NAME, "createFamilyMember", [$("#name").val(), weight]);
    }
}

function onUserAccountCreated(p_user) {
    console.log("onUserAccountCreated()");
    parent.window.analytics.trackEvent('User', 'Create', 'Family Member');

    if (parent.getURLParameter("ftue", location) == "true") {
        parent.loadFrame("ftue", "nav-hidden=true");
    } else {
        parent.loadFrame("players");
    }
}

function checkLength(p_field, p_fieldTips, p_name, min, max) {
    console.log("checkLength()");
    if (p_field.val().length > max || p_field.val().length < min) {
        updateTips(p_field, p_fieldTips, "length_check_error", p_name, min, max);
        return false;
    }
    return true;
}

function checkRegexp(p_field, p_fieldTips, regexp, p_error) {
    console.log("checkRegexp()");
    if (!(regexp.test(p_field.val()))) {
        updateTips(p_field, p_fieldTips, p_error);
        return false;
    }
    return true;
}

function checkMaxValue(p_field, p_fieldTips, p_maxValue, p_error) {
    console.log("checkMaxValue()");
    if (p_field.val() > p_maxValue) {
        updateTips(p_field, p_fieldTips, p_error);
        return false;
    }
    return true;
}

function checkMinValue(p_field, p_fieldTips, p_minValue, p_error) {
    console.log("checkValue()");
    if (p_field.val() < p_minValue) {
        updateTips(p_field, p_fieldTips, p_error);
        return false;
    }
    return true;
}

function checkMatch(p_first, p_second, p_tips, p_message) {
    console.log("checkMatch()");
    if (p_first.val() != p_second.val()) {
        updateTips(p_first, p_tips, p_message);
        return false;
    }
    return true;
}

function updateTips(p_field, p_fieldTips, p_error) {
    var copy = null;
    copy = [].slice.call(arguments);
    copy.splice(0, 2);
    p_error = parent.Tgoma.LanguageManager.getString.apply(parent.Tgoma.LanguageManager, copy);

    console.log("updateTips(" + p_field + ", " + p_fieldTips + ", " + p_error + ")");

    p_fieldTips.text(p_error).addClass("tip-state-visible");
    p_field.addClass("input-state-error");
}

function onTermsAndConditionsClicked(evt) {
    console.log("[CreatePlayer.htm] onTermsAndConditionsClicked()");
    parent.loadOverlayFrame("privacyPolicy.htm");
}

function checkForEmojis(p_field, p_tips, p_message) {
    var containsEmoji = p_field.val().search(/([\uD800-\uDBFF][\uDC00-\uDFFF])/g) >= 0 ||
        p_field.val().search(/([\uE000-\uF8FF]|\uD83C[\uDF00-\uDFFF]|\uD83D[\uDC00-\uDDFF])/g) >= 0; //Trying to find if there are emojis
    if (containsEmoji) {
        updateTips(p_field, p_tips, p_message);
        return false;
    }
    return true;
}
