var CONTROLLER_ERROR_CABLE_A = 0x1;
var CONTROLLER_ERROR_CABLE_B = 0x2;
var CONTROLLER_ERROR_CABLE_AB = 3;
var CONTROLLER_ERROR_AUTH = 0x4;
var CONTROLLER_ERROR_CABLE_A_IRQ_FAULT = 0x8;
var CONTROLLER_ERROR_CABLE_B_IRQ_FAULT = 0x10;
var CONTROLLER_ERROR_HARDWARE_FAULT = 0x20;

var CONTROLLER_WARNING_SENSOR_1_MISALIGNED = 0x10000;
var CONTROLLER_WARNING_SENSOR_2_MISALIGNED = 0x20000;
var CONTROLLER_WARNING_SENSOR_3_MISALIGNED = 0x40000;
var CONTROLLER_WARNING_SENSOR_4_MISALIGNED = 0x80000;

var CONTROLLER_WARNING_CORRUPT_TABLES = 0x100000;
var CONTROLLER_WARNING_MODEL_SELECT_DISABLED = 0x200000;
var CONTROLLER_WARNING_NO_MODEL_SET = 0x400000;

var CONTROLLER_NOTICE_LOW_BATT_POWER_OFF = 0x100000000;
var CONTROLLER_NOTICE_INACTIVITY_POWER_OFF = 0x200000000;
var CONTROLLER_NOTICE_SANDBOX_MODE = 0x400000000;
var CONTROLLER_NOTICE_FIRMWARE_UPDATE_ACTIVE = 0x800000000;
var CONTROLLER_NOTICE_NO_REF_VALUES = 0x1000000000;
var CONTROLLER_NOTICE_WATCHDOG_RESET = 0x2000000000;

var m_usePartForLocalisation = true;

var _errorImageDictionary = {
	1: "cableAError-v2.png",
	2: "cableBError-v2.png",
	3: "cableABError-v2.png"
}

var _errorTextDictionary1 = {
	1: "error_0" ,
	2: "error_1" ,
	3: "error_0_and_1" ,
	4: "error_2",
	8: "error_3",
	16: "error_4",
	0x20: "error_5",
	0x10000: "error_10",
	0x20000: "error_11",
	0x40000: "error_12",
	0x80000: "error_13",
	0x100000: "error_14",
	0x200000: "error_15",
	0x400000: "error_16",


}

var _errorTextDictionary2 = {
	0x1: "error_20",
	0x2: "error_21",
	0x4: "error_22",
	0x8: "error_23"
};



var _errorTextDictionary6 = {
	0x1: "error_161",
	0x10: "error_162"
};

function initialiseErrorDialog(p_languageManagerPath)
{
	console.log("initialiseErrorDialog("+p_languageManagerPath+")");
	Tgoma.LanguageManager.initialise(null,p_languageManagerPath);
	m_usePartForLocalisation = false;
}

function showErrorDialogWithEnum(p_enum, p_errorList, p_imageLookupCallback) {
	console.log("showErrorDialogWithEnum(" + p_enum + "," + p_errorList + ")");

	var errorTextDictionary;

	if(p_errorList == 1 )
	{
		errorTextDictionary = _errorTextDictionary1;
	}
	else if(p_errorList == 2)
	{
		errorTextDictionary = _errorTextDictionary2;
	}
	else if(p_errorList == 6)
	{
		errorTextDictionary = _errorTextDictionary6;
	}
	else //No other errors have been added yet.
	{
		console.log("These errors have not been set up yet");
		return;
	}


	switch(p_enum) {
		case 1: {
			if(p_errorList == 1)
			{
				showErrorDialogWithImage(_errorImageDictionary[p_enum], p_imageLookupCallback);
			}
			/*else
			{
				showErrorDialogWithText(errorTextDictionary[p_enum]);
			}*/
			break;
		}
		case 2: {
			if(p_errorList == 1)
			{
				showErrorDialogWithImage(_errorImageDictionary[p_enum], p_imageLookupCallback);
			}
			/*else
			{
				showErrorDialogWithText(errorTextDictionary[p_enum]);
			}*/
			break;
		}
		case 3: {
			if(p_errorList == 1)
			{
				showErrorDialogWithImage(_errorImageDictionary[p_enum], p_imageLookupCallback);
			}
			/*else
			{
				showErrorDialogWithText(errorTextDictionary[p_enum]);
			}*/
			break;
		}
		case 4: {
			//showErrorDialogWithText(errorTextDictionary[p_enum]);
			break;
		}
	}
}

function showErrorDialogWithImage(p_imageTag, p_imageLookupCallback) {
	console.log("showErrorDialogWithImage(" + p_imageTag + ")")
	var tag;
	tag = p_imageTag.substr(0,p_imageTag.indexOf(".png"));
	tag = tag.substr(0,tag.indexOf("-v2"));
	if(m_usePartForLocalisation)
	{
		tag = parent.Tgoma.LanguageManager.getString(tag);
	}
	else
	{
		tag = Tgoma.LanguageManager.getCommonString(tag);
	}

	showDialogWithContents("<div class=\"boldText\"style=\"\">" + tag + "</div> <img style=\"display: block; margin: auto;\" src=" + p_imageLookupCallback(p_imageTag) + " />", 340, 410);
}

function showErrorDialogWithText(p_text) {
	console.log("showErrorDialogWithText(" + p_text + ")")
	if(m_usePartForLocalisation)
	{
		p_text = parent.Tgoma.LanguageManager.getString(p_text);
	}
	else
	{
		p_text = Tgoma.LanguageManager.getCommonString(p_text);
	}


	showDialogWithContents("<p>" + p_text + "</p>", 'auto', 'auto');
}

function showDialogWithContents(p_contents, p_width, p_height) {
	console.log("showDialogWithContents(" + p_contents + ")");

	$('.error-dialog').empty();

	$('.error-dialog').prepend(p_contents);

	$(".error-dialog").dialog({
	    width: p_width,
	    height: p_height,
	    modal: true,
        open:function(){
            $(".ui-dialog-titlebar-close").hide();
            $('.ui-dialog-titlebar').hide();
            //get the last overlay in the dom
            $dialogOverlay = $(".ui-widget-overlay").last();
            //remove any event handler bound to it.
            $dialogOverlay.unbind();
            $dialogOverlay.click(function(){
            //close the dialog whenever the overlay is clicked.
                closeErrorDialog();
            });
        }
  	});

	$('.error-dialog').show();

  	$(".error-dialog").dialog( "option", "position", { my: "center", at: "center", of: window } );
}

function closeErrorDialog() {
	if ($(".error-dialog").hasClass('ui-dialog-content')) {
		$(".error-dialog").dialog('close');
	}
}
