var Tgoma;
(function (Tgoma) {
    function init(p_mainElementId, p_callback, p_useActionbar) {
        var _this = this;
        if (p_useActionbar === void 0) { p_useActionbar = true; }
        if (!Tgoma.ApplicationUtils.DeviceUtils.isMobile.any()) {
            Tgoma.BatteryHelper.init();
            Tgoma.HighScoreHelper.init();
            Tgoma.TrampolineHelper.init();
            Tgoma.EventHelper.init();
            Tgoma.ErrorHelper.init();
            if (p_callback != null) {
                p_callback(null);
            }
            return;
        }
        Tgoma.BatteryHelper.init();
        Tgoma.ConnectionHelper.init();
        Tgoma.HighScoreHelper.init();
        Tgoma.TrampolineHelper.init();
        Tgoma.UserHelper.init();
        Tgoma.EventHelper.init();
        Tgoma.ErrorHelper.init();
        Tgoma.HRMHelper.init();
        Tgoma.LanguageManager.initialise("", null, onLanguageManagerInitialised);
        screen.lockOrientation('landscape');
        TgomaMessaging.Internal.ApplicationManagement.getGoogleAnalyticsTrackingId(this, onGoogleAnalyticsTrackingIdReturned);
        if (p_useActionbar) {
            Tgoma.ApplicationUtils.FileUtils.getParentDirectory(function (p_parentDirectory) {
                console.log("onParentLocationReturned(" + p_parentDirectory + ")");
                p_parentDirectory = p_parentDirectory + "parent/";
                var libariesLoaded = false;
                var parentManifestLoaded = false;
                Tgoma.ApplicationUtils.FileUtils.loadTgomaLibraries(p_parentDirectory, function () {
                    TgomaMessaging.Internal.ApplicationManagement.getRaygunApiKey(_this, function (p_data) {
                        onRaygunApiKeyReturned(p_data);
                        libariesLoaded = true;
                        if (parentManifestLoaded && p_callback != null) {
                            ActionBar.initialise(p_mainElementId, p_parentDirectory);
                            p_callback(parentManifest);
                        }
                    });
                });
                var parentManifest = new Tgoma.ManifestLoader.Manifest(p_parentDirectory + "manifest.xml", function () {
                    console.log("Parent manifest load complete");
                    for (var asset in parentManifest.m_assets) {
                        parentManifest.m_assets[asset].m_path = p_parentDirectory + parentManifest.m_assets[asset].m_path;
                    }
                    parentManifestLoaded = true;
                    if (libariesLoaded) {
                        ActionBar.initialise(p_mainElementId, p_parentDirectory);
                        p_callback(parentManifest);
                    }
                }, null);
            }, null);
        }
        else {
            Tgoma.LanguageManager.initialise("", null, function () {
                if (p_callback != null) {
                    p_callback(null);
                }
            });
        }
    }
    Tgoma.init = init;
    function initForParent(p_callback) {
        var helpers = {
            "LanguageManager": false,
            "UserHelper": false
        };
        function onHelperReady(helperType) {
            if (!helpers[helperType]) {
                helpers[helperType] = true;
            }
            if (helpers["LanguageManager"] && helpers["UserHelper"] && p_callback != null) {
                p_callback();
            }
        }
        ;
        Tgoma.BatteryHelper.init();
        Tgoma.ConnectionHelper.init();
        Tgoma.TrampolineHelper.init();
        Tgoma.EventHelper.init();
        Tgoma.ErrorHelper.init();
        Tgoma.HighScoreHelper.init();
        Tgoma.LanguageManager.init(onHelperReady, "", null);
        Tgoma.UserHelper.init(onHelperReady);
        Tgoma.HRMHelper.init();
        screen.lockOrientation('landscape');
        TgomaMessaging.Internal.ApplicationManagement.getGoogleAnalyticsTrackingId(this, onGoogleAnalyticsTrackingIdReturned);
    }
    Tgoma.initForParent = initForParent;
    function onLanguageManagerInitialised() {
    }
    function onRaygunApiKeyReturned(p_data) {
        console.log("onRaygunApiKeyReturned(", p_data, ")");
        Raygun.init(p_data["apiKey"], {
            disablePulse: true,
            disableAnonymousUserTracking: true
        }).attach();
        if (Tgoma.UserHelper.currentFamilyAccount.m_parseId != null) {
            Raygun.setUser(Tgoma.UserHelper.currentFamilyAccount.m_parseId, false, Tgoma.UserHelper.currentFamilyAccount.m_email, Tgoma.UserHelper.currentFamilyAccount.m_familyName);
        }
        Raygun.saveIfOffline(true);
    }
    function onGoogleAnalyticsTrackingIdReturned(p_data) {
        console.log("onGoogleAnalyticsTrackingIdReturned(", p_data, ")");
        window.analytics.startTrackerWithId(p_data["trackingId"]);
        if (Tgoma.UserHelper.currentFamilyAccount.m_parseId != null) {
            window.analytics.setUserId(Tgoma.UserHelper.currentFamilyAccount.m_parseId);
        }
        document.addEventListener("onGoogleAnalyticsEvent", function (p_event) { return function (p_event) {
            if (p_event.event == "AppSpeed") {
                window.analytics.trackTiming("Parent", Number(p_event.label), "FirmwareKBytesTransferred");
            }
            else {
                window.analytics.trackEvent(p_event.event, p_event.action, p_event.label);
            }
        }; });
        document.addEventListener("onGoalProgress", function (p_event) { return function (p_event) {
            if (p_event.percentComplete >= 1.0) {
                window.analytics.trackEvent("Goal", "Complete");
            }
        }; });
        document.addEventListener("onControllerError", function (e) { return Tgoma.ErrorHelper.analyticsOnControllerFlag(e); }, false);
        window.analytics.trackEvent("tgomaStartup", "Language", navigator.language);
        window.analytics.trackEvent("tgomaStartup", "ProcessorArchitecture", navigator.platform);
    }
})(Tgoma || (Tgoma = {}));
var SignalBinding = (function () {
    function SignalBinding(signal, listener, isOnce, listenerContext, priority) {
        if (priority === void 0) { priority = 0; }
        this.active = true;
        this.params = null;
        this._listener = listener;
        this._isOnce = isOnce;
        this.context = listenerContext;
        this._signal = signal;
        this.priority = priority || 0;
    }
    SignalBinding.prototype.execute = function (paramsArr) {
        var handlerReturn;
        var params;
        if (this.active && !!this._listener) {
            params = this.params ? this.params.concat(paramsArr) : paramsArr;
            handlerReturn = this._listener.apply(this.context, params);
            if (this._isOnce) {
                this.detach();
            }
        }
        return handlerReturn;
    };
    SignalBinding.prototype.detach = function () {
        return this.isBound() ? this._signal.remove(this._listener, this.context) : null;
    };
    SignalBinding.prototype.isBound = function () {
        return (!!this._signal && !!this._listener);
    };
    SignalBinding.prototype.isOnce = function () {
        return this._isOnce;
    };
    SignalBinding.prototype.getListener = function () {
        return this._listener;
    };
    SignalBinding.prototype.getSignal = function () {
        return this._signal;
    };
    SignalBinding.prototype._destroy = function () {
        delete this._signal;
        delete this._listener;
        delete this.context;
    };
    SignalBinding.prototype.toString = function () {
        return '[SignalBinding isOnce:' + this._isOnce + ', isBound:' + this.isBound() + ', active:' + this.active + ']';
    };
    return SignalBinding;
}());
var Signal = (function () {
    function Signal() {
        this._bindings = [];
        this._prevParams = null;
        this.memorize = false;
        this._shouldPropagate = true;
        this.active = true;
    }
    Signal.prototype.validateListener = function (listener, fnName) {
        if (typeof listener !== 'function') {
            throw new Error('listener is a required param of {fn}() and should be a Function.'.replace('{fn}', fnName));
        }
    };
    Signal.prototype._registerListener = function (listener, isOnce, listenerContext, priority) {
        var prevIndex = this._indexOfListener(listener, listenerContext);
        var binding;
        if (prevIndex !== -1) {
            binding = this._bindings[prevIndex];
            if (binding.isOnce() !== isOnce) {
                throw new Error('You cannot add' + (isOnce ? '' : 'Once') + '() then add' + (!isOnce ? '' : 'Once') + '() the same listener without removing the relationship first.');
            }
        }
        else {
            binding = new SignalBinding(this, listener, isOnce, listenerContext, priority);
            this._addBinding(binding);
        }
        if (this.memorize && this._prevParams) {
            binding.execute(this._prevParams);
        }
        return binding;
    };
    Signal.prototype._addBinding = function (binding) {
        var n = this._bindings.length;
        do {
            --n;
        } while (this._bindings[n] && binding.priority <= this._bindings[n].priority);
        this._bindings.splice(n + 1, 0, binding);
    };
    Signal.prototype._indexOfListener = function (listener, context) {
        var n = this._bindings.length;
        var cur;
        while (n--) {
            cur = this._bindings[n];
            if (cur.getListener() === listener && cur.context === context) {
                return n;
            }
        }
        return -1;
    };
    Signal.prototype.has = function (listener, context) {
        if (context === void 0) { context = null; }
        return this._indexOfListener(listener, context) !== -1;
    };
    Signal.prototype.add = function (listener, listenerContext, priority) {
        if (listenerContext === void 0) { listenerContext = null; }
        if (priority === void 0) { priority = 0; }
        this.validateListener(listener, 'add');
        return this._registerListener(listener, false, listenerContext, priority);
    };
    Signal.prototype.addOnce = function (listener, listenerContext, priority) {
        if (listenerContext === void 0) { listenerContext = null; }
        if (priority === void 0) { priority = 0; }
        this.validateListener(listener, 'addOnce');
        return this._registerListener(listener, true, listenerContext, priority);
    };
    Signal.prototype.remove = function (listener, context) {
        if (context === void 0) { context = null; }
        this.validateListener(listener, 'remove');
        var i = this._indexOfListener(listener, context);
        if (i !== -1) {
            this._bindings[i]._destroy();
            this._bindings.splice(i, 1);
        }
        return listener;
    };
    Signal.prototype.removeAll = function () {
        var n = this._bindings.length;
        while (n--) {
            this._bindings[n]._destroy();
        }
        this._bindings.length = 0;
    };
    Signal.prototype.getNumListeners = function () {
        return this._bindings.length;
    };
    Signal.prototype.halt = function () {
        this._shouldPropagate = false;
    };
    Signal.prototype.dispatch = function () {
        var paramsArr = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            paramsArr[_i] = arguments[_i];
        }
        if (!this.active) {
            return;
        }
        var n = this._bindings.length;
        var bindings;
        if (this.memorize) {
            this._prevParams = paramsArr;
        }
        if (!n) {
            return;
        }
        bindings = this._bindings.slice(0);
        this._shouldPropagate = true;
        do {
            n--;
        } while (bindings[n] && this._shouldPropagate && bindings[n].execute(paramsArr) !== false);
    };
    Signal.prototype.forget = function () {
        this._prevParams = null;
    };
    Signal.prototype.dispose = function () {
        this.removeAll();
        delete this._bindings;
        delete this._prevParams;
    };
    Signal.prototype.toString = function () {
        return '[Signal active:' + this.active + ' numListeners:' + this.getNumListeners() + ']';
    };
    Signal.VERSION = '1.0.0';
    return Signal;
}());
var Tgoma;
(function (Tgoma) {
    var ManifestLoader;
    (function (ManifestLoader) {
        var Manifest = (function () {
            function Manifest(p_pathToManifestXML, p_parseComplete, p_context) {
                var _this = this;
                this.m_assets = {};
                this.m_groups = {};
                var _xmlDocument;
                var _xmlHTTPRequest = new XMLHttpRequest();
                _xmlHTTPRequest.open("GET", p_pathToManifestXML);
                _xmlHTTPRequest.send();
                _xmlHTTPRequest.onload = function (ev) {
                    _xmlDocument = _xmlHTTPRequest.responseXML;
                    _this.parseXML(_xmlDocument);
                    p_parseComplete.apply(p_context);
                };
            }
            Manifest.prototype.parseXML = function (_xmlDocument) {
                var _resourceList = _xmlDocument.querySelectorAll("resource");
                for (var _resourceIndex = 0; _resourceIndex < _resourceList.length; _resourceIndex++) {
                    var node = _resourceList[_resourceIndex];
                    var ass = new ManifestAsset(node);
                    this.m_assets[ass.m_tag] = ass;
                }
                var _groupList = _xmlDocument.querySelectorAll("group");
                for (var _groupIndex = 0; _groupIndex < _groupList.length; _groupIndex++) {
                    var _groupName = _groupList[_groupIndex].attributes['name'].value;
                    this.m_groups[_groupName] = new Array();
                    var groupAssets = _groupList[_groupIndex].childNodes;
                    this.getAllAssetsInGroup(groupAssets, _groupName);
                }
                var _data = _xmlDocument.getElementsByTagName("data");
                this.m_version = _data[0].attributes.getNamedItem("v").value;
                console.log(this.m_groups);
            };
            Object.defineProperty(Manifest.prototype, "version", {
                get: function () {
                    return this.m_version;
                },
                enumerable: true,
                configurable: true
            });
            Manifest.prototype.getAllAssetsInGroup = function (p_parentGroup, p_groupName) {
                var _childNodes;
                for (var _nodeIndex = 0; _nodeIndex < p_parentGroup.length; _nodeIndex++) {
                    var _node = p_parentGroup[_nodeIndex];
                    if (_node.nodeName == "#text") {
                        continue;
                    }
                    else if (_node.nodeName == "reference") {
                        this.addNodeToGroups(_node, p_groupName);
                    }
                    else if (_node.nodeName == "referencedGroup" && _node.hasChildNodes) {
                        this.getAllAssetsInGroup(_node.childNodes, p_groupName);
                    }
                }
            };
            Manifest.prototype.addNodeToGroups = function (p_node, p_groupName) {
                var tag = p_node.attributes['tag'].value;
                tag = tag.substring(tag.lastIndexOf("\/") + 1, tag.length);
                this.m_groups[p_groupName].push(tag);
            };
            Manifest.prototype.getAssetPathByTag = function (p_tag) {
                return this.m_assets[p_tag].m_path;
            };
            return Manifest;
        }());
        ManifestLoader.Manifest = Manifest;
        var ManifestAsset = (function () {
            function ManifestAsset(p_node) {
                this.m_tag = "";
                this.m_path = "";
                this.m_handler = "";
                this.m_size = 0;
                var _tag = this.getValue(p_node, 'tag');
                this.m_tag = _tag.substring(_tag.lastIndexOf("\/") + 1, _tag.length);
                this.m_path = _tag;
                this.m_handler = this.getValue(p_node, 'handler');
                this.m_size = this.getIntValue(p_node, 'size');
            }
            ManifestAsset.prototype.getValue = function (p_node, p_value) {
                var val = p_node.attributes[p_value];
                return val == null ? "" : val.value;
            };
            ManifestAsset.prototype.getIntValue = function (p_node, p_value) {
                var val = p_node.attributes[p_value];
                return val == null ? 0 : parseInt(val.value);
            };
            return ManifestAsset;
        }());
        ManifestLoader.ManifestAsset = ManifestAsset;
    })(ManifestLoader = Tgoma.ManifestLoader || (Tgoma.ManifestLoader = {}));
})(Tgoma || (Tgoma = {}));
var Tgoma;
(function (Tgoma) {
    var Display;
    (function (Display) {
        var TextStyle = (function () {
            function TextStyle() {
                this.size = 40;
                this.emphasis = "bold";
                this.fontFace = "Arial";
                this.fill = "#000000";
                this.align = "left";
                this.stroke = "#000000";
                this.strokeThickness = 0;
                this.wordWrap = true;
                this.wordWrapLength = 100;
            }
            Object.defineProperty(TextStyle.prototype, "font", {
                get: function () {
                    return ((this.emphasis || "") + " " + this.size + "px " + this.fontFace);
                },
                enumerable: true,
                configurable: true
            });
            TextStyle.prototype.clone = function () {
                var _copy = new TextStyle();
                var t_this = this;
                for (var property in t_this) {
                    _copy[property] = t_this[property];
                }
                return _copy;
            };
            return TextStyle;
        }());
        Display.TextStyle = TextStyle;
    })(Display = Tgoma.Display || (Tgoma.Display = {}));
})(Tgoma || (Tgoma = {}));
var Tgoma;
(function (Tgoma) {
    function loadJSON(jsonFile, p_context, callback) {
        var xobj = new XMLHttpRequest();
        xobj.open('GET', jsonFile, true);
        xobj.onreadystatechange = function () {
            if (xobj.readyState == 4 && xobj.status == 200) {
                callback(xobj.responseText, p_context);
            }
        };
        xobj.send(null);
    }
    var MissionManager = (function () {
        function MissionManager() {
            if (MissionManager._sInstance) {
                throw new Error("Error: Instantiation failed: Use MissionManager.getInstance() instead of new.");
            }
            this._mStartedMissionsList = new Array();
            this._mMissionsCompletedThisSession = new Array();
            this._mMissionCompeteSignal = new Signal();
            this._mNewMissionsList = new Array();
        }
        Object.defineProperty(MissionManager, "instance", {
            get: function () {
                if (MissionManager._sInstance === null) {
                    MissionManager._sInstance = new MissionManager();
                }
                return MissionManager._sInstance;
            },
            enumerable: true,
            configurable: true
        });
        MissionManager.prototype.initialise = function (p_missionsData) {
            this.parseMissionsJSON(p_missionsData, this);
            Tgoma.UserHelper.userChangedSignal.add(this.onUserChanged, this);
        };
        MissionManager.prototype.loadMissions = function () {
            TgomaMessaging.Game.Mission.loadMissions(this, this.onMissionsLoaded);
        };
        MissionManager.prototype.onUserChanged = function () {
            window.setTimeout(function () { MissionManager.instance.setupUserMissions(); }, 200);
        };
        MissionManager.prototype.loadStartedMissionsList = function () {
            TgomaMessaging.Game.Mission.loadStartedMissionsList(this, this.onStartedMissionsListLoaded);
        };
        MissionManager.prototype.onMissionsLoaded = function (p_missions, p_context) {
            if (p_context._mActiveMissionsList == null) {
                p_context._mActiveMissionsList = new Array();
            }
            else {
                p_context._mActiveMissionsList.length = 0;
            }
            var missionsList = JSON.parse(p_missions);
            for (var i in missionsList) {
                p_context._mActiveMissionsList.push(Tgoma.Mission.fromObject(missionsList[i]));
            }
        };
        MissionManager.prototype.setupUserMissions = function () {
            if (this._mActiveMissionsList == null) {
                this._mActiveMissionsList = new Array();
            }
            this.saveMissions();
            this.loadStartedMissionsList();
            this.loadMissions();
            while (this._mActiveMissionsList.length < this._mNumberOfActiveMissions) {
                if (!this.startNextMission()) {
                    return;
                }
            }
        };
        MissionManager.prototype.onStartedMissionsListLoaded = function (p_completedMissions, p_context) {
            p_context._mStartedMissionsList = JSON.parse(p_completedMissions);
        };
        MissionManager.prototype.setNumberOfActiveMissions = function (p_value) {
            this._mNumberOfActiveMissions = p_value;
        };
        MissionManager.prototype.addMission = function (p_missionTag, p_objectives, p_description, p_rewardType, p_rewardAmount) {
            return 0;
        };
        MissionManager.prototype.saveMissions = function () {
            for (var i in this._mActiveMissionsList) {
                this._mActiveMissionsList[i].save();
            }
        };
        MissionManager.prototype.startNextMission = function () {
            for (var i = 0; i < this._mMissionsList.length; i++) {
                if (this._mStartedMissionsList.indexOf(this._mMissionsList[i].missionTag) < 0) {
                    var mission = this._mMissionsList[i];
                    var missionID = this.addMission(mission.missionTag, mission.objectives, mission.description, mission.rewardType, mission.rewardQuantity);
                    mission.missionID = missionID;
                    this._mActiveMissionsList.push(mission);
                    this._mStartedMissionsList.push(mission.missionTag);
                    this._mNewMissionsList.push(mission);
                    return true;
                }
            }
            return false;
        };
        MissionManager.prototype.updateMissionList = function () {
            this._mNewMissionsList.length = 0;
            while (this._mActiveMissionsList.length < this._mNumberOfActiveMissions) {
                if (!this.startNextMission()) {
                    return;
                }
            }
            console.log(this._mNewMissionsList);
        };
        Object.defineProperty(MissionManager.prototype, "activeMissionsList", {
            get: function () {
                return this._mActiveMissionsList;
            },
            enumerable: true,
            configurable: true
        });
        MissionManager.prototype.parseMissionsJSON = function (p_missions, p_context) {
            var missionSettings = p_missions;
            p_context.setNumberOfActiveMissions(missionSettings.numberOfActiveMissions);
            if (this._mMissionsList == null) {
                p_context._mMissionsList = new Array();
            }
            else {
                p_context._mMissionsList.length = 0;
            }
            for (var i in missionSettings.Missions) {
                p_context._mMissionsList.push(new Tgoma.Mission(missionSettings.Missions[i].Tag, missionSettings.Missions[i].Description, missionSettings.Missions[i].MissionType, missionSettings.Missions[i].Target, missionSettings.Missions[i].MissionSubType, missionSettings.Missions[i].RewardType, missionSettings.Missions[i].RewardQuantity, missionSettings.Missions[i].Image));
            }
        };
        MissionManager.prototype.addCompletedMission = function (p_mission) {
            console.log(this._mMissionCompeteSignal.getNumListeners().toString());
            this._mMissionsCompletedThisSession.push(p_mission);
            this._mMissionCompeteSignal.dispatch(p_mission);
            console.log(this._mMissionsCompletedThisSession);
        };
        Object.defineProperty(MissionManager.prototype, "newMissionsList", {
            get: function () {
                return this._mNewMissionsList;
            },
            enumerable: true,
            configurable: true
        });
        MissionManager.prototype.removeAllCompleteMissions = function () {
            console.log(this._mMissionsCompletedThisSession);
            console.log("Removing completed missions");
            for (var i = this._mActiveMissionsList.length - 1; i >= 0; i--) {
                console.log("checking mission", this._mActiveMissionsList[i]);
                if (this._mActiveMissionsList[i].status == Tgoma.Mission.MISSION_COMPLETE) {
                    console.log("Removing mission");
                    this._mActiveMissionsList.splice(i, 1);
                }
            }
            console.log(this._mActiveMissionsList);
        };
        MissionManager.prototype.removeActiveMission = function (p_mission) {
            this._mActiveMissionsList.splice(this._mActiveMissionsList.indexOf(p_mission), 1);
        };
        Object.defineProperty(MissionManager.prototype, "missionsCompletedThisSession", {
            get: function () {
                return this._mMissionsCompletedThisSession;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(MissionManager.prototype, "missionCompleteSignal", {
            get: function () {
                return this._mMissionCompeteSignal;
            },
            enumerable: true,
            configurable: true
        });
        MissionManager._sInstance = null;
        return MissionManager;
    }());
    Tgoma.MissionManager = MissionManager;
})(Tgoma || (Tgoma = {}));
var Tgoma;
(function (Tgoma) {
    var Mission = (function () {
        function Mission(p_tag, p_description, p_missionType, p_target, p_subType, p_rewardType, p_rewardQuantity, p_image) {
            this.missionTag = p_tag;
            this.description = p_description;
            this.objectives = new Array();
            this.objectives.push(new Tgoma.MissionGoal(p_missionType, p_target, p_subType, 0, this));
            this.rewardType = p_rewardType;
            this.rewardQuantity = p_rewardQuantity;
            this.image = p_image;
            this.status = Mission.MISSION_ACTIVE;
        }
        Mission.fromObject = function (jsonObject) {
            var mission = new Mission();
            mission.missionID = jsonObject["missionID"];
            mission.missionTag = jsonObject["missionTag"];
            mission.userName = jsonObject["userName"];
            mission.gameID = jsonObject["gameID"];
            mission.rewardType = jsonObject["rewardType"];
            mission.rewardQuantity = jsonObject["rewardQuantity"];
            mission.startDate = jsonObject["startDate"];
            mission.endDate = jsonObject["endDate"];
            mission.status = jsonObject["status"];
            mission.description = jsonObject["description"];
            mission.objectives = new Array();
            var objectives = jsonObject["objectives"];
            for (var i in objectives) {
                mission.objectives.push(Tgoma.MissionGoal.fromObject(objectives[i], mission));
            }
            return mission;
        };
        Mission.prototype.save = function () {
            for (var i in this.objectives) {
                TgomaMessaging.Game.Mission.saveMissionGoal(this.objectives[i].goalID, this.objectives[i].progress);
            }
            TgomaMessaging.Game.Mission.saveMission(this.missionID, this.status, this.description);
        };
        Object.defineProperty(Mission.prototype, "completed", {
            get: function () {
                for (var i in this.objectives) {
                    if (!this.objectives[i].completed) {
                        return false;
                    }
                }
                return true;
            },
            enumerable: true,
            configurable: true
        });
        Mission.prototype.goalCompleted = function () {
            if (this.completed && this.status == Mission.MISSION_ACTIVE) {
                console.log("MissionComplete");
                Tgoma.MissionManager.instance.addCompletedMission(this);
                this.status = Mission.MISSION_COMPLETE;
                this.endDate = new Date().toTimeString();
                this.save();
                console.log(this);
            }
        };
        Mission.MISSION_ACTIVE = 1;
        Mission.MISSION_COMPLETE = 2;
        return Mission;
    }());
    Tgoma.Mission = Mission;
})(Tgoma || (Tgoma = {}));
var Tgoma;
(function (Tgoma) {
    var MissionGoal = (function () {
        function MissionGoal(p_goalType, p_target, p_subType, p_goalID, p_parentMission) {
            if (p_subType === void 0) { p_subType = 0; }
            if (p_goalID === void 0) { p_goalID = 0; }
            this.GoalType = p_goalType;
            this.Target = p_target;
            this.GoalID = p_goalID;
            this.parentMission = p_parentMission;
            this.Progress = 0;
            this.SubType = p_subType;
        }
        Object.defineProperty(MissionGoal.prototype, "goalType", {
            get: function () {
                return this.GoalType;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(MissionGoal.prototype, "target", {
            get: function () {
                return this.Target;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(MissionGoal.prototype, "goalID", {
            get: function () {
                return this.GoalID;
            },
            set: function (p_value) {
                this.GoalID = p_value;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(MissionGoal.prototype, "goalSubType", {
            get: function () {
                return this.SubType;
            },
            enumerable: true,
            configurable: true
        });
        MissionGoal.prototype.addOne = function (p_subType) {
            console.log("adding one");
            if (this.SubType == undefined || this.SubType == null || p_subType == this.SubType || this.SubType < 0) {
                this.progress++;
            }
        };
        MissionGoal.prototype.setMax = function (p_value, p_subType) {
            if (this.SubType == undefined || this.SubType == null || p_subType == this.SubType || this.SubType < 0) {
                if (p_value > this.Progress) {
                    this.progress = p_value;
                }
            }
        };
        MissionGoal.prototype.addProgress = function (p_progress, p_subType) {
            if (this.SubType == undefined || this.SubType == null || p_subType == this.SubType || this.SubType < 0) {
                this.progress += p_progress;
            }
        };
        MissionGoal.prototype.checkGreaterThanSub = function (p_value) {
            if (p_value >= this.SubType) {
                this.addOne(this.SubType);
            }
        };
        MissionGoal.prototype.checkLessThanSub = function (p_value) {
            if (p_value <= this.SubType) {
                this.addOne(this.SubType);
            }
        };
        Object.defineProperty(MissionGoal.prototype, "progress", {
            get: function () {
                return this.Progress;
            },
            set: function (p_value) {
                if (this.completed)
                    return;
                this.Progress = p_value;
                console.log("Adding progress " + this.Progress + " of " + this.target);
                if (this.completed) {
                    this.Progress = this.Target;
                    this.parentMission.goalCompleted();
                }
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(MissionGoal.prototype, "completed", {
            get: function () {
                return this.Progress >= this.Target;
            },
            enumerable: true,
            configurable: true
        });
        MissionGoal.fromObject = function (jsonObject, p_mission) {
            var goal = new MissionGoal(jsonObject.GoalType, jsonObject.Target, jsonObject.SubType, jsonObject.GoalID, p_mission);
            goal.Progress = jsonObject.Progress;
            console.log("Current progress = " + goal.progress);
            return goal;
        };
        return MissionGoal;
    }());
    Tgoma.MissionGoal = MissionGoal;
})(Tgoma || (Tgoma = {}));
var Tgoma;
(function (Tgoma) {
    var Goal = (function () {
        function Goal() {
        }
        return Goal;
    }());
    Tgoma.Goal = Goal;
})(Tgoma || (Tgoma = {}));
var Tgoma;
(function (Tgoma) {
    var UserHelper = (function () {
        function UserHelper() {
        }
        UserHelper.init = function (p_callback) {
            if (p_callback === void 0) { p_callback = null; }
            UserHelper.m_readyCallback = p_callback;
            document.addEventListener("onFamilyMemberChanged", function (e) { return UserHelper.onGetFamilyMemberChanged(e); }, false);
            document.addEventListener("onFamilyMemberUpdated", function (e) { return UserHelper.onFamilyMemberUpdated(e); }, false);
            document.addEventListener("onFamilyLoginReturned", function (e) { return UserHelper.onFamilyLoginReturned(e); }, false);
            document.addEventListener("onFamilyCreateReturned", function (e) { return UserHelper.onFamilyCreateReturned(e); }, false);
            document.addEventListener("onFamilyAccountUpdated", function (e) { return UserHelper.onFamilyAccountUpdated(e); }, false);
            TgomaMessaging.Internal.UserManagement.getFamilyAccount(this, UserHelper.onGetFamilyAccountReturned);
            TgomaMessaging.Internal.UserManagement.getFamilyMember(this, UserHelper.onGetFamilyMemberReturned);
        };
        UserHelper.destroy = function () {
            document.removeEventListener("onFamilyMemberChanged", function () { return UserHelper.onGetFamilyMemberChanged; });
            document.removeEventListener("onFamilyMemberUpdated", function () { return UserHelper.onFamilyMemberUpdated; });
            document.removeEventListener("onFamilyLoginReturned", function () { return UserHelper.onFamilyLoginReturned; });
            document.removeEventListener("onFamilyCreateReturned", function () { return UserHelper.onFamilyCreateReturned; });
            document.removeEventListener("onFamilyAccountUpdated", function () { return UserHelper.onFamilyAccountUpdated; });
        };
        UserHelper.onGetFamilyAccountReturned = function (p_familyAccountJSON) {
            console.log("onGetFamilyAccountReturned()");
            UserHelper.m_currentFamilyAccount = new FamilyAccount(p_familyAccountJSON);
            if (UserHelper.m_readyCallback != null && UserHelper.m_currentFamilyMember != null) {
                UserHelper.m_readyCallback("UserHelper");
                UserHelper.m_readyCallback = null;
            }
        };
        UserHelper.onFamilyAccountUpdated = function (p_event) {
            console.log("onFamilyAccountUpdated()");
            if (p_event.success) {
                UserHelper.m_currentFamilyAccount = new FamilyAccount(p_event.family);
            }
        };
        UserHelper.onGetFamilyMemberReturned = function (p_familyMemberJSON) {
            console.log("onGetFamilyMemberReturned()");
            UserHelper.m_currentFamilyMember = new FamilyMember(p_familyMemberJSON);
            UserHelper.m_userChangedSignal.dispatch(UserHelper.m_currentFamilyMember);
            if (UserHelper.m_readyCallback != null && UserHelper.m_currentFamilyAccount != null) {
                UserHelper.m_readyCallback("UserHelper");
                UserHelper.m_readyCallback = null;
            }
        };
        UserHelper.onGetFamilyMemberChanged = function (p_event) {
            console.log("onGetFamilyMemberChanged()");
            UserHelper.m_currentFamilyMember = new FamilyMember(p_event.familyMember);
            UserHelper.m_userChangedSignal.dispatch(UserHelper.m_currentFamilyMember);
        };
        UserHelper.onFamilyMemberUpdated = function (p_event) {
            console.log("onFamilyMemberUpdated()");
            UserHelper.m_currentFamilyMember = new FamilyMember(p_event.familyMember);
            UserHelper.m_userUpdatedSignal.dispatch(UserHelper.m_currentFamilyMember);
        };
        Object.defineProperty(UserHelper, "currentUser", {
            get: function () {
                return UserHelper.currentFamilyMemberName;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(UserHelper, "currentFamilyMemberName", {
            get: function () {
                return UserHelper.m_currentFamilyMember.m_name;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(UserHelper, "userChangedSignal", {
            get: function () {
                return UserHelper.m_userChangedSignal;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(UserHelper, "userUpdatedSignal", {
            get: function () {
                return UserHelper.m_userUpdatedSignal;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(UserHelper, "currentFamilyMember", {
            get: function () {
                return UserHelper.m_currentFamilyMember;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(UserHelper, "currentFamilyAccount", {
            get: function () {
                return UserHelper.m_currentFamilyAccount;
            },
            enumerable: true,
            configurable: true
        });
        UserHelper.onUserCreationComplete = function (p_outcome, p_displayNameOrError) {
            console.log("onUserCreationComplete(" + p_outcome + ", " + p_displayNameOrError + ")");
            UserHelper.m_userCreationCompleteSignal.dispatch(p_outcome, p_displayNameOrError);
        };
        Object.defineProperty(UserHelper, "userCreationCompleteSignal", {
            get: function () {
                console.log("userCreationCompleteSignal()");
                return UserHelper.m_userCreationCompleteSignal;
            },
            enumerable: true,
            configurable: true
        });
        UserHelper.onFamilyCreateReturned = function (p_event) {
            console.log("onFamilyCreateReturned(" + p_event.succeeded + ", " + p_event.errorOrId + ")");
            UserHelper.m_familyCreationReturnedSignal.dispatch(p_event.succeeded, p_event.errorOrId);
        };
        UserHelper.onFamilyLoginReturned = function (p_event) {
            console.log("onFamilyLoginReturned(" + p_event.succeeded + ", " + p_event.errorOrId + ")");
            UserHelper.m_familyLoginReturnedSignal.dispatch(p_event.succeeded, p_event.errorOrId);
        };
        Object.defineProperty(UserHelper, "familyCreationReturnedSignal", {
            get: function () {
                console.log("familyCreationReturnedSignal()");
                return UserHelper.m_familyCreationReturnedSignal;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(UserHelper, "familyLoginReturnedSignal", {
            get: function () {
                console.log("familyCreationReturnedSignal()");
                return UserHelper.m_familyLoginReturnedSignal;
            },
            enumerable: true,
            configurable: true
        });
        UserHelper.m_userChangedSignal = new Signal();
        UserHelper.m_userUpdatedSignal = new Signal();
        UserHelper.m_userCreationCompleteSignal = new Signal();
        UserHelper.m_familyCreationReturnedSignal = new Signal();
        UserHelper.m_familyLoginReturnedSignal = new Signal();
        return UserHelper;
    }());
    Tgoma.UserHelper = UserHelper;
    var FamilyAccount = (function () {
        function FamilyAccount(p_jsonObject) {
            this.m_familyName = p_jsonObject["familyName"];
            this.m_country = p_jsonObject["country"];
            this.m_region = p_jsonObject["region"];
            this.m_email = p_jsonObject["email"];
            this.m_dateIsUSFormat = p_jsonObject["dateIsUSFormat"];
            this.m_energyIsUSFormat = p_jsonObject["energyIsUSFormat"];
            this.m_unitsAreUSFormat = p_jsonObject["unitsAreUSFormat"];
            this.m_parseId = p_jsonObject["parseId"];
            this.m_serialNumber = p_jsonObject["serialNumber"];
        }
        return FamilyAccount;
    }());
    Tgoma.FamilyAccount = FamilyAccount;
    var FamilyMember = (function () {
        function FamilyMember(p_jsonObject) {
            this.m_name = p_jsonObject["userName"];
            this.m_weight = p_jsonObject["weight"];
            this.m_isVerified = p_jsonObject["isVerified"];
        }
        return FamilyMember;
    }());
    Tgoma.FamilyMember = FamilyMember;
})(Tgoma || (Tgoma = {}));
var Tgoma;
(function (Tgoma) {
    var ConnectionHelper = (function () {
        function ConnectionHelper() {
        }
        ConnectionHelper.init = function () {
            document.addEventListener("onConnectionStatusChanged", function (e) { return ConnectionHelper.onConnectionStatusChanged(e); }, false);
            TgomaMessaging.Misc.getIsConnectedToTrampoline(this, ConnectionHelper.onConnectionStatusReturned);
        };
        ConnectionHelper.destroy = function () {
            document.removeEventListener("onConnectionStatusChanged");
        };
        ConnectionHelper.onConnectionStatusChanged = function (p_event) {
            console.log("onConnectionStatusChanged(" + p_event.connectionStatus + ")");
            ConnectionHelper._mConnected = p_event.connectionStatus;
            ConnectionHelper.m_connectionChangedSignal.dispatch(ConnectionHelper._mConnected);
            if (!p_event.connectionStatus) {
                Tgoma.ErrorHelper.onDisconnect();
            }
        };
        ConnectionHelper.onConnectionStatusReturned = function (p_connectionStatus) {
            console.log("onConnectionStatusChanged(" + p_connectionStatus + ")");
            ConnectionHelper._mConnected = p_connectionStatus;
            ConnectionHelper.m_connectionChangedSignal.dispatch(ConnectionHelper._mConnected);
        };
        Object.defineProperty(ConnectionHelper, "isConnected", {
            get: function () {
                return ConnectionHelper._mConnected;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ConnectionHelper, "connectionSignal", {
            get: function () {
                return ConnectionHelper.m_connectionChangedSignal;
            },
            enumerable: true,
            configurable: true
        });
        ConnectionHelper._mConnected = false;
        ConnectionHelper.m_connectionChangedSignal = new Signal();
        return ConnectionHelper;
    }());
    Tgoma.ConnectionHelper = ConnectionHelper;
})(Tgoma || (Tgoma = {}));
var Tgoma;
(function (Tgoma) {
    var TrampolineHelper = (function () {
        function TrampolineHelper() {
        }
        TrampolineHelper.init = function () {
            document.addEventListener("onGenericTrampolineMessage", function (e) { return TrampolineHelper.onGenericMessage(e); }, false);
            document.addEventListener("onPolarLocationMessage", function (e) { return TrampolineHelper.onPolarLocationMessage(e); }, false);
            document.addEventListener("onTrampolineDetailsUpdated", function (e) { return TrampolineHelper.onTrampolineDetailsUpdated(e); }, false);
        };
        TrampolineHelper.destroy = function () {
            document.removeEventListener("onGenericTrampolineMessage", function () { return TrampolineHelper.onGenericMessage; });
            document.removeEventListener("onPolarLocationMessage", function () { return TrampolineHelper.onPolarLocationMessage; });
            document.removeEventListener("onTrampolineDetailsUpdated", function () { return TrampolineHelper.onTrampolineDetailsUpdated; });
        };
        TrampolineHelper.onGenericMessage = function (p_event) {
            console.log("[TrampolineHelper] [TS] onGenericMessage(" + p_event.eventId + ", " + p_event.payload + ")");
            if (TrampolineHelper.callbacks[p_event.eventId] != null) {
                TrampolineHelper.callbacks[p_event.eventId].dispatch(p_event.payload);
            }
        };
        TrampolineHelper.onPolarLocationMessage = function (p_event) {
            console.log("[TrampolineHelper] [TS] onPolarLocationMessage(" + p_event.magnitude + ", " + p_event.angle + ")");
            TrampolineHelper.m_gameAreaNormValue = p_event.unitCircleNorm;
            TrampolineHelper.m_maximumAreaNormValue = p_event.maximisedAreaNorm;
            TrampolineHelper.m_polarLocationDeprecatedSignal.dispatch(p_event.magnitude, p_event.angle);
            var angle_rads = (Math.PI / 128) * ((64 - p_event.angle) % 256);
            var unitCircleRadius = Math.min(p_event.magnitude / p_event.unitCircleNorm, 1);
            var maxAreaNormRadius = Math.min(p_event.magnitude / p_event.maximisedAreaNorm, 1);
            TrampolineHelper.m_polarLocationUnitCircleNormSignal.dispatch(unitCircleRadius, angle_rads);
            TrampolineHelper.m_polarLocationMaximumAreaNormSignal.dispatch(maxAreaNormRadius, angle_rads);
        };
        TrampolineHelper.onTrampolineDetailsUpdated = function (p_event) {
            console.log("[TrampolineHelper] [TS] onTrampolineDetailsUpdated(" + p_event.trampolineModel + ", " + p_event.hardwareVersion + ", " + p_event.firmwareVersion + ", " + p_event.hardwareId + ")");
            TrampolineHelper.m_firmwareVersion = p_event.firmwareVersion;
            TrampolineHelper.m_trampolineDetailsChangedSignal.dispatch(p_event.hardwareId, p_event.trampolineModel, p_event.hardwareVersion, p_event.firmwareVersion);
            if (p_event.firmwareVersion != "" && p_event.firmwareVersion != "unset") {
                window.analytics.trackEvent("FirmwareVersion", p_event.firmwareVersion);
            }
        };
        TrampolineHelper.subscribeToMessage = function (p_eventID, p_callback, p_callbackContext) {
            console.log("[TrampolineHelper] [TS] subscribeToMessage(" + p_eventID + ")");
            if (TrampolineHelper.callbacks[p_eventID] == null) {
                TrampolineHelper.callbacks[p_eventID] = new Signal();
            }
            TrampolineHelper.callbacks[p_eventID].add(p_callback, p_callbackContext);
        };
        TrampolineHelper.unsubscribeFromMessage = function (p_eventID, p_callback, p_callbackContext) {
            if (TrampolineHelper.callbacks[p_eventID] != null) {
                TrampolineHelper.callbacks[p_eventID].remove(p_callback, p_callbackContext);
            }
        };
        Object.defineProperty(TrampolineHelper, "polarLocationMessageDeprecatedSignal", {
            get: function () {
                return TrampolineHelper.m_polarLocationDeprecatedSignal;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(TrampolineHelper, "polarLocationGameCircleNormalSignal", {
            get: function () {
                return TrampolineHelper.m_polarLocationUnitCircleNormSignal;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(TrampolineHelper, "polarLocationMaximumAreaSignal", {
            get: function () {
                return TrampolineHelper.m_polarLocationMaximumAreaNormSignal;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(TrampolineHelper, "trampolineDetailsChangedSignal", {
            get: function () {
                return TrampolineHelper.m_trampolineDetailsChangedSignal;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(TrampolineHelper, "trampolineGameAreaNormal", {
            get: function () {
                return TrampolineHelper.m_gameAreaNormValue;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(TrampolineHelper, "trampolineMaximumAreaNormal", {
            get: function () {
                return TrampolineHelper.m_maximumAreaNormValue;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(TrampolineHelper, "allTrampolineModels", {
            get: function () {
                console.log("allTrampolineModels()");
                var trampolineModels = [];
                trampolineModels.push(new TrampolineModel(1, "trampoline_r54"));
                trampolineModels.push(new TrampolineModel(2, "trampoline_r79"));
                trampolineModels.push(new TrampolineModel(3, "trampoline_o77"));
                trampolineModels.push(new TrampolineModel(4, "trampoline_o92"));
                trampolineModels.push(new TrampolineModel(5, "trampoline_s113"));
                trampolineModels.push(new TrampolineModel(6, "trampoline_s155"));
                trampolineModels.push(new TrampolineModel(7, "trampoline_r132"));
                return trampolineModels;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(TrampolineHelper, "supportedTrampolineModels", {
            get: function () {
                console.log("supportedTrampolineModels()");
                console.log("Firmware Version =" + TrampolineHelper.m_firmwareVersion);
                var indexOfFirstPoint = TrampolineHelper.m_firmwareVersion.indexOf(".");
                var firmwareVersionMajor = parseInt(TrampolineHelper.m_firmwareVersion.substring(0, indexOfFirstPoint));
                var firmwareVersionMinor = parseInt(TrampolineHelper.m_firmwareVersion.substring(indexOfFirstPoint + 1, TrampolineHelper.m_firmwareVersion.indexOf(".", indexOfFirstPoint + 1)));
                var trampolineModels = [];
                trampolineModels.push(new TrampolineModel(1, "trampoline_r54"));
                trampolineModels.push(new TrampolineModel(2, "trampoline_r79"));
                if (firmwareVersionMajor > 1 || firmwareVersionMinor >= 1) {
                    trampolineModels.push(new TrampolineModel(3, "trampoline_o77"));
                    trampolineModels.push(new TrampolineModel(4, "trampoline_o92"));
                }
                trampolineModels.push(new TrampolineModel(5, "trampoline_s113"));
                trampolineModels.push(new TrampolineModel(6, "trampoline_s155"));
                if (firmwareVersionMajor > 1 || firmwareVersionMinor >= 2) {
                    trampolineModels.push(new TrampolineModel(7, "trampoline_r132"));
                }
                return trampolineModels;
            },
            enumerable: true,
            configurable: true
        });
        TrampolineHelper.m_polarLocationDeprecatedSignal = new Signal();
        TrampolineHelper.m_polarLocationUnitCircleNormSignal = new Signal();
        TrampolineHelper.m_polarLocationMaximumAreaNormSignal = new Signal();
        TrampolineHelper.callbacks = {};
        TrampolineHelper.m_trampolineDetailsChangedSignal = new Signal();
        TrampolineHelper.m_gameAreaNormValue = 0;
        TrampolineHelper.m_maximumAreaNormValue = 0;
        TrampolineHelper.m_firmwareVersion = "unset";
        return TrampolineHelper;
    }());
    Tgoma.TrampolineHelper = TrampolineHelper;
    var TrampolineModel = (function () {
        function TrampolineModel(p_value, p_tag) {
            this.tag = p_tag;
            this.value = p_value;
        }
        return TrampolineModel;
    }());
    Tgoma.TrampolineModel = TrampolineModel;
})(Tgoma || (Tgoma = {}));
var Tgoma;
(function (Tgoma) {
    var ErrorHelper = (function () {
        function ErrorHelper() {
        }
        ErrorHelper.init = function () {
            document.addEventListener("onControllerError", function (e) { return ErrorHelper.onControllerError(e); }, false);
        };
        ErrorHelper.destroy = function () {
            document.removeEventListener("onControllerError", function (e) { return ErrorHelper.onControllerError(e); });
            document.removeEventListener("onControllerError", function (e) { return ErrorHelper.analyticsOnControllerFlag(e); });
        };
        ErrorHelper.onDisconnect = function () {
            ErrorHelper.m_noIdleRef = false;
            ErrorHelper.m_corruptLUT = false;
        };
        ErrorHelper.onControllerError = function (p_event) {
            console.log("onControllerError(" + p_event.errorField1 + ", " + +p_event.errorField2 + ", " + p_event.errorField3 + ")");
            ErrorHelper.m_errorEnum1 = p_event.errorField1;
            ErrorHelper.m_errorEnum2 = p_event.errorField2;
            ErrorHelper.m_errorEnum3 = p_event.errorField3;
            ErrorHelper.m_errorSignal.dispatch(p_event.errorField1, p_event.errorField2, p_event.errorField3);
        };
        ErrorHelper.analyticsOnControllerFlag = function (p_event) {
            if (p_event.errorField1 > 0) {
                var w = 4294967296;
                var noRefFlag = CONTROLLER_NOTICE_NO_REF_VALUES / w;
                var watchdogFlag = CONTROLLER_NOTICE_WATCHDOG_RESET / w;
                var errorHigh = p_event.errorField1 / w;
                var errorLow = p_event.errorField1 % w;
                if ((noRefFlag & errorHigh)) {
                    if (!ErrorHelper.m_noIdleRef) {
                        ErrorHelper.m_noIdleRef = true;
                        window.analytics.trackEvent("tgomaStartup", "FirmwareFlags", "No Idle Reference Values");
                    }
                }
                else if (ErrorHelper.m_noIdleRef) {
                    ErrorHelper.m_noIdleRef = false;
                }
                if ((watchdogFlag & errorHigh)) {
                    window.analytics.trackEvent("tgomaStartup", "FirmwareFlags", "Watchdog Reset");
                }
                if ((CONTROLLER_WARNING_CORRUPT_TABLES & errorLow)) {
                    if (!ErrorHelper.m_corruptLUT) {
                        ErrorHelper.m_corruptLUT = true;
                        window.analytics.trackEvent("tgomaStartup", "FirmwareFlags", "Corrupt Lookup Tables");
                        window.analytics.trackEvent("tgomaStartup", "tableCorruptLUT");
                    }
                }
                else if (ErrorHelper.m_corruptLUT) {
                    ErrorHelper.m_corruptLUT = false;
                }
            }
        };
        ;
        Object.defineProperty(ErrorHelper, "errorSignal", {
            get: function () {
                return ErrorHelper.m_errorSignal;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ErrorHelper, "currentErrorField1", {
            get: function () {
                return ErrorHelper.m_errorEnum1;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ErrorHelper, "currentErrorField2", {
            get: function () {
                return ErrorHelper.m_errorEnum2;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ErrorHelper, "currentErrorField3", {
            get: function () {
                return ErrorHelper.m_errorEnum3;
            },
            enumerable: true,
            configurable: true
        });
        ErrorHelper.m_errorSignal = new Signal();
        ErrorHelper.m_errorEnum1 = 0;
        ErrorHelper.m_errorEnum2 = 0;
        ErrorHelper.m_errorEnum3 = 0;
        ErrorHelper.m_noIdleRef = false;
        ErrorHelper.m_corruptLUT = false;
        return ErrorHelper;
    }());
    Tgoma.ErrorHelper = ErrorHelper;
})(Tgoma || (Tgoma = {}));
var Tgoma;
(function (Tgoma) {
    var EventHelper = (function () {
        function EventHelper() {
        }
        EventHelper.init = function () {
            document.addEventListener("onEventLeaderboardReturned", function (e) { return EventHelper.onEventLeaderboardReturned(e); }, false);
            document.addEventListener("onEventLeaderboardUserRankReturned", function (e) { return EventHelper.onEventLeaderboardUserRankReturned(e); }, false);
        };
        EventHelper.destroy = function () {
            document.removeEventListener("onEventLeaderboardReturned", EventHelper.onEventLeaderboardReturned);
            document.removeEventListener("onEventLeaderboardUserRankReturned", EventHelper.onEventLeaderboardUserRankReturned);
        };
        EventHelper.onEventLeaderboardReturned = function (p_event) {
            console.log("onEventLeaderboardReturned(" + p_event.eventLeaderboard + ")");
            EventHelper.m_eventLeaderboardReturnedSignal.dispatch(JSON.parse(p_event.eventLeaderboard));
        };
        EventHelper.onEventLeaderboardUserRankReturned = function (p_event) {
            console.log("onEventLeaderboardUserRankReturned(" + p_event.eventLeaderboardUserRank + ")");
            EventHelper.m_eventLeaderboardUserRankReturnedSignal.dispatch(JSON.parse(p_event.eventLeaderboardUserRank));
        };
        Object.defineProperty(EventHelper, "eventLeaderboardReturnedSignal", {
            get: function () {
                return EventHelper.m_eventLeaderboardReturnedSignal;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(EventHelper, "userRankReturnedSignal", {
            get: function () {
                return EventHelper.m_eventLeaderboardUserRankReturnedSignal;
            },
            enumerable: true,
            configurable: true
        });
        EventHelper.m_eventLeaderboardReturnedSignal = new Signal();
        EventHelper.m_eventLeaderboardUserRankReturnedSignal = new Signal();
        return EventHelper;
    }());
    Tgoma.EventHelper = EventHelper;
})(Tgoma || (Tgoma = {}));
var Tgoma;
(function (Tgoma) {
    var HighScoreHelper = (function () {
        function HighScoreHelper() {
        }
        HighScoreHelper.init = function () {
            document.addEventListener("onHighScoreReturned", function (e) { return HighScoreHelper.onHighScoreReturned; }, false);
            document.addEventListener("onUserRankReturned", function (e) { return HighScoreHelper.onUserRankReturned; }, false);
            document.addEventListener("onHighscoreError", function (e) { return HighScoreHelper.onHighScoreError; }, false);
        };
        HighScoreHelper.getLeaderboard = function (p_local, p_level) {
            HighScoreHelper.m_leaderboard = null;
            HighScoreHelper.m_userRank = null;
            HighScoreHelper.m_requestedLevel = p_level;
            if (p_local) {
                TgomaMessaging.Game.Leaderboard.getLocalLeaderboardDataForLevel(p_level);
            }
            else {
                TgomaMessaging.Game.Leaderboard.getLeaderboardDataForLevel(p_level);
            }
        };
        HighScoreHelper.destroy = function () {
            document.removeEventListener("onHighScoreReturned", function () { return HighScoreHelper.onHighScoreReturned; });
            document.removeEventListener("onUserRankReturned", function () { return HighScoreHelper.onUserRankReturned; });
            document.removeEventListener("onHighscoreError", function () { return HighScoreHelper.onHighScoreError; });
        };
        HighScoreHelper.onHighScoreReturned = function (p_event) {
            console.log("onHighScoreReturned(" + p_event.highScores + ")");
            if (p_event.level == HighScoreHelper.m_requestedLevel) {
                HighScoreHelper.m_leaderboard = JSON.parse(p_event.highScores);
                if (HighScoreHelper.m_userRank != null) {
                    HighScoreHelper.m_leaderboardReturnedSignal.dispatch(new LeaderboardData(HighScoreHelper.m_leaderboard, HighScoreHelper.m_userRank));
                }
            }
        };
        HighScoreHelper.onHighScoreError = function (p_event) {
            console.log("onHighScoreError(" + p_event.errorMessage + "," + p_event.errorId + ")");
            HighScoreHelper.m_leaderboardErrorSignal.dispatch(p_event.errorMessage, p_event.errorId);
        };
        HighScoreHelper.onUserRankReturned = function (p_event) {
            console.log("onUserRankReturned(" + p_event.userRank + ")");
            if (p_event.level == HighScoreHelper.m_requestedLevel) {
                HighScoreHelper.m_userRank = JSON.parse(p_event.userRank);
                if (HighScoreHelper.m_leaderboard != null) {
                    HighScoreHelper.m_leaderboardReturnedSignal.dispatch(new LeaderboardData(HighScoreHelper.m_leaderboard, HighScoreHelper.m_userRank));
                }
            }
        };
        Object.defineProperty(HighScoreHelper, "leaderboardReturnedSignal", {
            get: function () {
                return HighScoreHelper.m_leaderboardReturnedSignal;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(HighScoreHelper, "leadboardErrorSignal", {
            get: function () {
                return HighScoreHelper.m_leaderboardErrorSignal;
            },
            enumerable: true,
            configurable: true
        });
        HighScoreHelper.m_leaderboard = null;
        HighScoreHelper.m_userRank = null;
        HighScoreHelper.m_requestedLevel = 0;
        HighScoreHelper.m_leaderboardReturnedSignal = new Signal();
        HighScoreHelper.m_leaderboardErrorSignal = new Signal();
        return HighScoreHelper;
    }());
    Tgoma.HighScoreHelper = HighScoreHelper;
    var LeaderboardData = (function () {
        function LeaderboardData(p_leaderboard, p_userRank) {
            this.leaderboard = p_leaderboard;
            this.userRank = p_userRank;
        }
        return LeaderboardData;
    }());
    Tgoma.LeaderboardData = LeaderboardData;
    var HighScoreData = (function () {
        function HighScoreData(p_name, p_rank, p_score, p_userId, p_country) {
            this.m_name = p_name;
            this.m_rank = p_rank;
            this.m_score = p_score;
            this.m_userId = p_userId;
            this.m_country = p_country;
        }
        Object.defineProperty(HighScoreData.prototype, "name", {
            get: function () {
                return this.m_name;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(HighScoreData.prototype, "userId", {
            get: function () {
                return this.m_userId;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(HighScoreData.prototype, "rank", {
            get: function () {
                return this.m_rank;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(HighScoreData.prototype, "score", {
            get: function () {
                return this.m_score;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(HighScoreData.prototype, "country", {
            get: function () {
                return this.m_country;
            },
            enumerable: true,
            configurable: true
        });
        return HighScoreData;
    }());
    Tgoma.HighScoreData = HighScoreData;
})(Tgoma || (Tgoma = {}));
var Tgoma;
(function (Tgoma) {
    var BatteryHelper = (function () {
        function BatteryHelper() {
        }
        BatteryHelper.init = function () {
            document.addEventListener("onBatteryStatus", function (e) { return BatteryHelper.onBatteryStatus(e); }, false);
        };
        BatteryHelper.destroy = function () {
            document.removeEventListener("onBatteryStatus", function (e) { return BatteryHelper.onBatteryStatus(e); });
        };
        BatteryHelper.onBatteryStatus = function (p_event) {
            console.log("onBatteryStatus(" + p_event.status + ")");
            BatteryHelper.m_batteryHelper.dispatch(p_event.status);
        };
        Object.defineProperty(BatteryHelper, "batterySignal", {
            get: function () {
                return BatteryHelper.m_batteryHelper;
            },
            enumerable: true,
            configurable: true
        });
        BatteryHelper.m_batteryHelper = new Signal();
        return BatteryHelper;
    }());
    Tgoma.BatteryHelper = BatteryHelper;
})(Tgoma || (Tgoma = {}));
var Tgoma;
(function (Tgoma) {
    var Utils = (function () {
        function Utils() {
        }
        Utils.centerWithinBoundsV = function (p_object, p_top, p_bottom, p_proportion) {
            if (p_proportion === void 0) { p_proportion = 0.5; }
            p_object.position.y = Math.round(p_top + ((p_bottom - p_top) * p_proportion) - (p_object.height / 2));
        };
        Utils.centerWithinBoundsH = function (p_object, p_left, p_right, p_proportion) {
            if (p_proportion === void 0) { p_proportion = 0.5; }
            p_object.position.x = Math.round(p_left + ((p_right - p_left) * p_proportion) - (p_object.width / 2));
        };
        Utils.roundedRandomInRange = function (p_min, p_max) {
            return Math.floor(Utils.randomInRange(p_min, p_max));
        };
        Utils.randomInRange = function (p_min, p_max) {
            return (Math.random() * (p_max - p_min + 1)) + p_min;
        };
        Utils.convertMetresToFeet = function (m_metres) {
            return m_metres * 3.28084;
        };
        Utils.convertFeetToMetres = function (m_feet) {
            return m_feet * 0.3048;
        };
        Utils.convertKilogramsToPounds = function (m_kilograms) {
            return m_kilograms * 2.20462;
        };
        Utils.convertPoundsToKilograms = function (m_pounds) {
            return m_pounds * 0.453592;
        };
        Utils.convertCaloriesToKiloJoules = function (m_calories) {
            return m_calories * 4.184;
        };
        Utils.convertKiloJoulesToCalories = function (m_kilojoules) {
            return m_kilojoules * 0.239;
        };
        return Utils;
    }());
    Tgoma.Utils = Utils;
})(Tgoma || (Tgoma = {}));
var Tgoma;
(function (Tgoma) {
    var LanguageManager = (function () {
        function LanguageManager() {
        }
        LanguageManager.init = function (p_callback, p_directoryPath, p_commonDirectoryPath) {
            if (p_directoryPath === void 0) { p_directoryPath = null; }
            if (p_commonDirectoryPath === void 0) { p_commonDirectoryPath = null; }
            console.log("LanguageManager - Initialise");
            LanguageManager.m_languageFiles = new Object();
            LanguageManager.m_commonLanguageFiles = new Object();
            var language = LanguageManager.determineLanguage();
            if (p_directoryPath != null) {
                $.get(p_directoryPath + language + ".lang", function (data, textStatus, jqXHR) {
                    var lines = data.split(/\n/g);
                    for (var i in lines) {
                        var line = lines[i];
                        var indexOfComma = line.indexOf(",");
                        var tag = line.substr(0, indexOfComma);
                        var text = line.substr(indexOfComma + 1);
                        text = LanguageManager.replaceAll(text, "\\n", '\n');
                        LanguageManager.m_languageFiles[tag] = text;
                    }
                    LanguageManager.m_language = language;
                    LanguageManager.m_ready = true;
                    if (p_callback != null) {
                        p_callback("LanguageManager");
                    }
                });
            }
            if (p_commonDirectoryPath != null) {
                $.get(p_commonDirectoryPath + language + ".lang", function (data, textStatus, jqXHR) {
                    var lines = data.split(/\n/g);
                    for (var i in lines) {
                        var line = lines[i];
                        var indexOfComma = line.indexOf(",");
                        var tag = line.substr(0, indexOfComma);
                        var text = line.substr(indexOfComma + 1);
                        text = LanguageManager.replaceAll(text, "\\n", '\n');
                        LanguageManager.m_commonLanguageFiles[tag] = text;
                    }
                    LanguageManager.m_language = language;
                    LanguageManager.m_ready = true;
                    if (p_callback != null) {
                        p_callback("LanguageManager");
                    }
                });
            }
        };
        LanguageManager.determineLanguage = function () {
            var deviceLanguage = navigator.language;
            console.log("LanguageManager - device language is " + deviceLanguage);
            var languageTag = deviceLanguage.substr(0, 2);
            LanguageManager.m_languageTag = languageTag;
            var language = "english";
            if (languageTag == "en") {
                language = "english";
            }
            else if (languageTag == "fr") {
                language = "french";
            }
            else if (languageTag == "de") {
                language = "german";
            }
            else if (languageTag == "ru") {
                language = "russian";
            }
            else if (languageTag == "zh") {
                language = "chinese";
            }
            return language;
        };
        LanguageManager.showTags = function (p_showTags) {
            LanguageManager.m_showTags = p_showTags;
        };
        LanguageManager.getString = function (p_tag) {
            var params = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                params[_i - 1] = arguments[_i];
            }
            var text = LanguageManager.m_languageFiles[p_tag];
            if (text != null && params != null) {
                if (params.length >= 1) {
                    if (text.indexOf("<x>") >= 0) {
                        text = text.replace("<x>", params[0]);
                    }
                    else {
                        console.log("Error finding <x>");
                    }
                }
                if (params.length >= 2) {
                    if (text.indexOf("<y>") >= 0) {
                        text = text.replace("<y>", params[1]);
                    }
                    else {
                        console.log("Error finding <y>");
                    }
                }
                if (params.length >= 3) {
                    if (text.indexOf("<z>") >= 0) {
                        text = text.replace("<z>", params[2]);
                    }
                    else {
                        console.log("Error finding <z>");
                    }
                }
            }
            return text != null && !LanguageManager.m_showTags ? text : p_tag;
        };
        LanguageManager.getCommonString = function (p_tag) {
            var params = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                params[_i - 1] = arguments[_i];
            }
            var text = LanguageManager.m_commonLanguageFiles[p_tag];
            if (text != null && params != null) {
                if (params.length >= 1) {
                    if (text.indexOf("<x>") >= 0) {
                        text = text.replace("<x>", params[0]);
                    }
                    else {
                        console.log("Error finding <x>");
                    }
                }
                if (params.length >= 2) {
                    if (text.indexOf("<y>") >= 0) {
                        text = text.replace("<y>", params[1]);
                    }
                    else {
                        console.log("Error finding <y>");
                    }
                }
                if (params.length >= 3) {
                    if (text.indexOf("<z>") >= 0) {
                        text = text.replace("<z>", params[2]);
                    }
                    else {
                        console.log("Error finding <z>");
                    }
                }
            }
            return text != null && !LanguageManager.m_showTags ? text : p_tag;
        };
        LanguageManager.getLanguage = function () {
            return LanguageManager.m_language;
        };
        LanguageManager.getLanguageTag = function () {
            return LanguageManager.m_languageTag;
        };
        LanguageManager.replaceAll = function (input, search, replacement) {
            return input.split(search).join(replacement);
        };
        LanguageManager.initialise = function (p_directoryPath, p_commonDirectoryPath, p_callback) {
            if (p_directoryPath === void 0) { p_directoryPath = null; }
            if (p_commonDirectoryPath === void 0) { p_commonDirectoryPath = null; }
            if (p_callback === void 0) { p_callback = null; }
            console.log("LanguageManager - Initialise");
            if (p_directoryPath != null) {
                LanguageManager.m_baseDirectory = p_directoryPath;
            }
            if (p_commonDirectoryPath != null) {
                LanguageManager.m_commonBaseDirectory = p_commonDirectoryPath;
            }
            LanguageManager.m_languageFiles = new Object();
            LanguageManager.m_commonLanguageFiles = new Object();
            LanguageManager.setLanguage(LanguageManager.determineLanguage(), p_callback);
        };
        LanguageManager.setLanguage = function (p_language, p_callback) {
            if (LanguageManager.m_baseDirectory != null) {
                $.get(LanguageManager.m_baseDirectory + p_language + ".lang", function (data, textStatus, jqXHR) {
                    var lines = data.split(/\n/g);
                    for (var i in lines) {
                        var line = lines[i];
                        var indexOfComma = line.indexOf(",");
                        var tag = line.substr(0, indexOfComma);
                        var text = line.substr(indexOfComma + 1);
                        text = LanguageManager.replaceAll(text, "\\n", '\n');
                        LanguageManager.m_languageFiles[tag] = text;
                    }
                    LanguageManager.m_language = p_language;
                    LanguageManager.m_ready = true;
                    if (p_callback != null) {
                        p_callback();
                    }
                });
            }
            if (LanguageManager.m_commonBaseDirectory != null) {
                $.get(LanguageManager.m_commonBaseDirectory + p_language + ".lang", function (data, textStatus, jqXHR) {
                    var lines = data.split(/\n/g);
                    for (var i in lines) {
                        var line = lines[i];
                        var indexOfComma = line.indexOf(",");
                        var tag = line.substr(0, indexOfComma);
                        var text = line.substr(indexOfComma + 1);
                        text = LanguageManager.replaceAll(text, "\\n", '\n');
                        LanguageManager.m_commonLanguageFiles[tag] = text;
                    }
                    LanguageManager.m_language = p_language;
                    LanguageManager.m_ready = true;
                    if (p_callback != null) {
                        p_callback();
                    }
                });
            }
        };
        LanguageManager.setLanguageFileDirectory = function (p_directory) {
            LanguageManager.m_baseDirectory = p_directory;
            LanguageManager.setLanguage("english", null);
        };
        LanguageManager.m_baseDirectory = null;
        LanguageManager.m_commonBaseDirectory = null;
        LanguageManager.m_showTags = false;
        LanguageManager.m_ready = false;
        return LanguageManager;
    }());
    Tgoma.LanguageManager = LanguageManager;
})(Tgoma || (Tgoma = {}));
var HRM_SERVICE_UUID = "180D";
var HRM_HR_MEASUREMENT_CHARACTERISTIC_UUID = "2A37";
var Tgoma;
(function (Tgoma) {
    var HRMHelper = (function () {
        function HRMHelper() {
        }
        HRMHelper.init = function () {
            console.log("HRMHelper Initialise()");
            var promise = new window.Promise(function (resolve, reject) {
                bluetoothle.initialize(resolve, { request: false, statusReceiver: false });
            }).then(HRMHelper.initializeSuccess, HRMHelper.handleError);
        };
        HRMHelper.initializeSuccess = function (result) {
            if (result.status === "enabled") {
                console.log("Bluetooth is enabled.");
                console.log(result);
                HRMHelper.isScanEnabled = true;
                HRMHelper.IsBluetoothEnabled = true;
            }
            else {
                console.log("Bluetooth is not enabled:", "status");
                console.log(result, "status");
                HRMHelper.isScanEnabled = false;
            }
        };
        HRMHelper.destroy = function () {
        };
        HRMHelper.startScan = function () {
            HRMHelper.foundDevices = [];
            if (this.isScanEnabled) {
                console.log("Starting scan for devices...", "status");
                bluetoothle.startScan(HRMHelper.startScanSuccess, HRMHelper.handleError, { services: [HRM_SERVICE_UUID] });
            }
            else {
                console.log("Scanning is disabled!", "status");
            }
        };
        HRMHelper.startScanSuccess = function (result) {
            console.log("startScanSuccess(" + result.status + ")");
            if (result.status === "scanStarted") {
                console.log("Scanning for devices (will continue to scan until you select a device)...", "status");
            }
            else if (result.status === "scanResult") {
                if (!HRMHelper.foundDevices.some(function (device) {
                    return device.address === result.address;
                })) {
                    console.log('FOUND DEVICE:');
                    console.log(result);
                    HRMHelper.foundDevices.push(result);
                    HRMHelper.m_foundDevice.dispatch(result);
                }
            }
        };
        HRMHelper.stopScan = function () {
            console.log("Stopping scan for devices...", "status");
            new window.Promise(function (resolve, reject) {
                bluetoothle.stopScan(resolve, reject);
            }).then(null, HRMHelper.handleError);
        };
        HRMHelper.connect = function (address) {
            console.log('Connecting to device: ' + address + "...", "status");
            HRMHelper.attemptReconnect = true;
            new window.Promise(function (resolve, reject) {
                bluetoothle.connect(resolve, reject, { address: address });
            }).then(HRMHelper.connectSuccess, HRMHelper.handleConnectError);
            window.analytics.trackEvent("Heart Rate Monitor", "Connect");
        };
        HRMHelper.disconnect = function () {
            console.log("Disconnecting from device");
            HRMHelper.attemptReconnect = false;
            new window.Promise(function (resolve, reject) {
                bluetoothle.disconnect(resolve, reject, { address: HRMHelper.monitorAddress });
            }).then(HRMHelper.connectSuccess, HRMHelper.handleDisconnectError);
        };
        HRMHelper.connectSuccess = function (result) {
            console.log("- " + result.status);
            HRMHelper.failedConnectAttempts = 0;
            if (result.status === "connected") {
                HRMHelper.getDeviceServices(result.address);
                HRMHelper.IsConnected = true;
            }
            else if (result.status === "disconnected") {
                console.log("Disconnected from device: " + result.address, "status");
                HRMHelper.IsConnected = false;
            }
        };
        HRMHelper.handleConnectError = function (error) {
            var msg;
            if (error.error && error.message) {
                msg = "Error on " + error.error + ": " + error.message;
            }
            else {
                msg = error;
            }
            console.log(msg, "error");
            if (HRMHelper.failedConnectAttempts < 10 && HRMHelper.attemptReconnect) {
                HRMHelper.failedConnectAttempts++;
                new window.Promise(function (resolve, reject) {
                    bluetoothle.reconnect(resolve, reject, { address: error.address });
                }).then(HRMHelper.connectSuccess, HRMHelper.handleConnectError);
            }
        };
        HRMHelper.handleDisconnectError = function (error) {
            var msg;
            if (error.error && error.message) {
                msg = "Error on " + error.error + ": " + error.message;
            }
            else {
                msg = error;
            }
            console.log(msg, "error");
        };
        HRMHelper.getDeviceServices = function (address) {
            console.log("Getting device services...", "status");
            var platform = window.cordova.platformId;
            if (platform === "android") {
                new window.Promise(function (resolve, reject) {
                    bluetoothle.discover(resolve, reject, { address: address });
                }).then(HRMHelper.discoverSuccess, HRMHelper.handleError);
            }
            else if (platform === "ios") {
                new window.Promise(function (resolve, reject) {
                    bluetoothle.services(resolve, reject, { address: address, services: [HRM_SERVICE_UUID] });
                }).then(HRMHelper.iosDiscoverServiceSuccess, HRMHelper.handleError);
            }
            else {
                console.log("Unsupported platform: '" + window.cordova.platformId + "'", "error");
            }
        };
        HRMHelper.iosDiscoverServiceSuccess = function (result) {
            console.log("Discover returned with status: " + result.status);
            if (result.services.some(function (service) { return service === HRM_SERVICE_UUID; })) {
                new window.Promise(function (resolve, reject) {
                    bluetoothle.characteristics(resolve, reject, { address: result.address, service: HRM_SERVICE_UUID, characteristics: [HRM_HR_MEASUREMENT_CHARACTERISTIC_UUID] });
                }).then(HRMHelper.iosDiscoverCharacteristicSuccess, HRMHelper.handleError);
            }
            else {
                console.log("Device is not a Heart Rate Monitor");
                new window.Promise(function (resolve, reject) {
                    bluetoothle.disconnect(resolve, reject, { address: result.address });
                }).then(HRMHelper.connectSuccess, HRMHelper.handleError);
            }
        };
        HRMHelper.iosDiscoverCharacteristicSuccess = function (result) {
            console.log("Discover returned with status: " + result.status);
            if (result.characteristics.some(function (characteristic) { return characteristic.uuid === HRM_HR_MEASUREMENT_CHARACTERISTIC_UUID; })) {
                HRMHelper.addService(result.address);
            }
            else {
                console.log("Device is not a suitable Heart Rate Monitor");
                new window.Promise(function (resolve, reject) {
                    bluetoothle.disconnect(resolve, reject, { address: result.address });
                }).then(HRMHelper.connectSuccess, HRMHelper.handleError);
            }
        };
        HRMHelper.discoverSuccess = function (result) {
            console.log("Discover returned with status: " + result.status);
            if (result.services.some(function (service) { return service.uuid === HRM_SERVICE_UUID; })) {
                HRMHelper.addService(result.address);
            }
            else {
                console.log("Device is not a Heart Rate Monitor");
                new window.Promise(function (resolve, reject) {
                    bluetoothle.disconnect(resolve, reject, { address: result.address });
                }).then(HRMHelper.connectSuccess, HRMHelper.handleError);
            }
        };
        HRMHelper.addService = function (address) {
            console.log('Adding service');
            HRMHelper.monitorAddress = address;
            HRMHelper.subscribeToMeasurement();
        };
        HRMHelper.subscribeToMeasurement = function () {
            bluetoothle.subscribe(HRMHelper.subscribeSuccess, HRMHelper.handleError, { address: HRMHelper.monitorAddress, service: HRM_SERVICE_UUID, characteristic: HRM_HR_MEASUREMENT_CHARACTERISTIC_UUID });
        };
        HRMHelper.subscribeSuccess = function (result) {
            console.log("subscribeSuccess():");
            console.log(result);
            if (result.status === "subscribedResult") {
                console.log("service: " + result.service + " characteristic: " + result.characteristic + "value: " + result.value);
                HRMHelper.readMeasurementResult(result);
            }
            if (result.status === "subscribed") {
                console.log("service: " + result.service + " characteristic: " + result.characteristic + "value: " + result.value);
            }
        };
        HRMHelper.readMeasurementResult = function (result) {
            var bytes = bluetoothle.encodedStringToBytes(result.value);
            if (bytes.length === 0) {
                return;
            }
            var flag = bytes[0];
            var offset = 1;
            if ((flag & 0x01) == 1) {
                var u16bytesHr = bytes.buffer.slice(offset, offset + 2);
                var u16Hr = new Uint16Array(u16bytesHr)[0];
                HRMHelper.m_hrMeasurement.dispatch(u8Hr);
                console.log("measurement: " + u16Hr.toString());
            }
            else {
                var u8bytesHr = bytes.buffer.slice(offset, offset + 1);
                var u8Hr = new Uint8Array(u8bytesHr)[0];
                HRMHelper.m_hrMeasurement.dispatch(u8Hr);
                console.log("measurement: " + u8Hr.toString());
            }
        };
        HRMHelper.handleError = function (error) {
            var msg;
            if (error.error && error.message) {
                var errorItems = [];
                if (error.service) {
                    errorItems.push("service: " + (error.service));
                }
                if (error.characteristic) {
                    errorItems.push("characteristic: " + (error.characteristic));
                }
                msg = "Error on " + error.error + ": " + error.message + (errorItems.length && (" (" + errorItems.join(", ") + ")"));
            }
            else {
                msg = error;
            }
            console.log(msg, "error");
            if (error.error == "isDisconnected" && HRMHelper.failedConnectAttempts < 10 && HRMHelper.attemptReconnect) {
                HRMHelper.m_hrMeasurement.dispatch(0);
                HRMHelper.connect(HRMHelper.monitorAddress);
            }
        };
        Object.defineProperty(HRMHelper, "foundDeviceSignal", {
            get: function () {
                return HRMHelper.m_foundDevice;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(HRMHelper, "hrMeasurementSignal", {
            get: function () {
                return HRMHelper.m_hrMeasurement;
            },
            enumerable: true,
            configurable: true
        });
        HRMHelper.isScanEnabled = false;
        HRMHelper.foundDevices = [];
        HRMHelper.failedConnectAttempts = 0;
        HRMHelper.attemptReconnect = false;
        HRMHelper.IsConnected = false;
        HRMHelper.IsBluetoothEnabled = false;
        HRMHelper.m_foundDevice = new Signal();
        HRMHelper.m_hrMeasurement = new Signal();
        return HRMHelper;
    }());
    Tgoma.HRMHelper = HRMHelper;
})(Tgoma || (Tgoma = {}));
var Tgoma;
(function (Tgoma) {
    var ApplicationUtils;
    (function (ApplicationUtils) {
        var FileUtils = (function () {
            function FileUtils() {
            }
            FileUtils.loadManifest = function (p_path, p_callback) {
                console.log("loadManifest(" + p_path + ")");
                var xmlhttp = new XMLHttpRequest();
                xmlhttp.open("GET", p_path, false);
                xmlhttp.onload = p_callback;
                xmlhttp.send();
            };
            FileUtils.loadCordovaLibraries = function (p_document) {
                if (DeviceUtils.isMobile.Android()) {
                    console.log("Running on Android");
                    var script = document.createElement('script');
                    script.type = 'text/javascript';
                    script.src = '/android_asset/www/cordova.js';
                    p_document.head.appendChild(script);
                }
                else if (DeviceUtils.isMobile.iOS()) {
                    console.log("Running on iOS");
                    var script = document.createElement('script');
                    script.type = 'text/javascript';
                    script.src = '../cordova.js';
                    p_document.head.appendChild(script);
                }
                else {
                    console.log("Running on Desktop");
                }
                var _errorElement = p_document.createElement("div");
                _errorElement.setAttribute("class", "error-dialog");
                p_document.body.appendChild(_errorElement);
            };
            FileUtils.loadTgomaLibraries = function (p_pathToParentApp, p_callback) {
                if (p_callback === void 0) { p_callback = null; }
                this.m_pathToParentApp = p_pathToParentApp;
                var errorDialogScript = document.createElement('script');
                errorDialogScript.type = 'text/javascript';
                errorDialogScript.src = p_pathToParentApp + "errorDialog-v2.js";
                errorDialogScript.onload = ApplicationUtils.FileUtils.onLibraryLoaded;
                document.head.appendChild(errorDialogScript);
                FileUtils.m_librariesToLoad++;
                var actionbarScript = document.createElement('script');
                actionbarScript.type = 'text/javascript';
                actionbarScript.src = p_pathToParentApp + "actionbar.js";
                actionbarScript.onload = ApplicationUtils.FileUtils.onLibraryLoaded;
                document.head.appendChild(actionbarScript);
                FileUtils.m_librariesToLoad++;
                var jqueryScript = document.createElement('script');
                jqueryScript.type = 'text/javascript';
                jqueryScript.src = p_pathToParentApp + "jquery-1.10.2.min.js";
                jqueryScript.onload = ApplicationUtils.FileUtils.onLibraryLoaded;
                document.head.appendChild(jqueryScript);
                FileUtils.m_librariesToLoad++;
                var jqueryUIScript = document.createElement('script');
                jqueryUIScript.type = 'text/javascript';
                jqueryUIScript.src = p_pathToParentApp + "jquery-ui.min.js";
                jqueryUIScript.onload = ApplicationUtils.FileUtils.onLibraryLoaded;
                document.head.appendChild(jqueryUIScript);
                FileUtils.m_librariesToLoad++;
                var jqueryCSS = document.createElement("link");
                jqueryCSS.rel = 'stylesheet';
                jqueryCSS.href = p_pathToParentApp + "jquery-ui.css";
                jqueryCSS.type = 'text/css';
                jqueryCSS.onload = ApplicationUtils.FileUtils.onLibraryLoaded;
                document.head.appendChild(jqueryCSS);
                FileUtils.m_librariesToLoad++;
                var raygunScript = document.createElement('script');
                raygunScript.type = 'text/javascript';
                raygunScript.src = p_pathToParentApp + "raygun.js";
                raygunScript.onload = ApplicationUtils.FileUtils.onLibraryLoaded;
                document.head.appendChild(raygunScript);
                FileUtils.m_librariesToLoad++;
                FileUtils.m_librariesCallback = p_callback;
            };
            FileUtils.onLibraryLoaded = function (p_event) {
                console.log("onLibraryLoaded()");
                if (++FileUtils.m_librariesLoaded == FileUtils.m_librariesToLoad && FileUtils.m_librariesCallback != null) {
                    console.log("onLibraryLoaded finished()");
                    initialiseErrorDialog(FileUtils.m_pathToParentApp);
                    FileUtils.m_librariesCallback();
                }
            };
            FileUtils.getPathByTagInXML = function (p_xml, p_tag) {
                console.log("getPathByTagInXML(" + p_tag + ")");
                return p_tag;
            };
            FileUtils.getParentDirectory = function (p_callback, p_context) {
                TgomaMessaging.Misc.getParentDirectory(p_context, p_callback);
            };
            FileUtils.m_librariesLoaded = 0;
            FileUtils.m_librariesToLoad = 0;
            FileUtils.m_pathToParentApp = "";
            return FileUtils;
        }());
        ApplicationUtils.FileUtils = FileUtils;
        var DeviceUtils = (function () {
            function DeviceUtils() {
            }
            DeviceUtils.isMobile = {
                Android: function () {
                    return navigator.userAgent.match(/Android/i);
                },
                iOS: function () {
                    return navigator.userAgent.match(/iPhone|iPad|iPod/i);
                },
                Windows: function () {
                    return navigator.userAgent.match(/IEMobile/i) || navigator.userAgent.match(/WPDesktop/i);
                },
                any: function () {
                    return (DeviceUtils.isMobile.Android() || DeviceUtils.isMobile.iOS() || DeviceUtils.isMobile.Windows());
                }
            };
            return DeviceUtils;
        }());
        ApplicationUtils.DeviceUtils = DeviceUtils;
    })(ApplicationUtils = Tgoma.ApplicationUtils || (Tgoma.ApplicationUtils = {}));
})(Tgoma || (Tgoma = {}));
