cordova.define("com.springfree.springfreegames.plugins.SpringfreeMessaging.TgomaMessaging", function(require, exports, module) {
exports.TGOMA_MODULE_NAME = "SpringfreeMessaging";
exports.GAME_MODULE_NAME = "Game";
exports.BLUETOOTH_MODULE_NAME = "Bluetooth";
exports.USER_MODULE_NAME = "UserManagement";
function callPlugin(p_success, p_error, p_context, p_module, p_method, p_args) {
    if (p_method != "poll") {
        console.log("[TgomaMessaging] [TS] callPlugin(" + JSON.stringify({ _module: p_module, _method: p_method, _args: p_args }, null, 4) + ")");
    }
    // Make sure the onSuccess and onError functions at least have a default callback!
    if (p_success) {
        p_success = p_success.bind(p_context);
    }
    else {
        p_success = onSuccess;
    }
    if (p_error) {
        p_error = p_error.bind(p_context);
    }
    else {
        p_error = onError;
    }
    // Call cordova.exec with:
    cordova.exec(
    // onSuccess - attempt to parse the JSON before giving it back to the /real/ onSuccess function
    function (p_payload) {
        console.log(p_payload);
        try {
            p_payload = JSON.parse(p_payload);
        }
        catch (exception) {
            console.log("[TgomaMessaging] [TS] callPlugin success exception: " + exception.toString());
        }
        finally {
            p_success(p_payload, p_context);
        }
    }, 
    // onError - attempt to parse the JSON before giving it back to the /real/ onError function
    function (p_payload) {
        console.log(p_payload);
        try {
            p_payload = JSON.parse(p_payload);
        }
        catch (exception) {
            console.log("[TgomaMessaging] [TS] callPlugin error exception: " + exception.toString());
        }
        finally {
            p_error(p_payload, p_context);
        }
    }, 
    /*
    The following line is a bit of a hack to maintain simplicity of code on the Java/Objective C receiving end of these messages.
    module and method is important information that needs to make it through to the C++. Cordova's objective C doesn't relay this information
    exactly; it directly pipes the call into a method on the corresponding module. This would require an empty function to be made in Objective C for every method we need in C++.
    We need to preserve the info, so we're going to push it to the front of the existing argument array - remembering to extract it again on the other side -
    And we're sending it to a 'fake' module/method combination "TgomaMessaging"/"TgomaMessaging".
    */
    exports.TGOMA_MODULE_NAME, exports.TGOMA_MODULE_NAME, [p_module, p_method, p_args]);
}
exports.callPlugin = callPlugin;
function onSuccess(p_payload, p_context) {
    console.log("[TgomaMessaging] [TS] onSuccess(" + JSON.stringify(p_payload, null, 4) + ")");
}
function onError(p_payload, p_context) {
    console.log("[TgomaMessaging] [TS] onError(" + JSON.stringify(p_payload, null, 4) + ")");
}
var Game;
(function (Game) {
    function returnToMainApplication() {
        callPlugin(null, null, this, exports.GAME_MODULE_NAME, "returnToMainApplication", []);
        Internal.ApplicationManagement.toggleScreenTimeout(false);
    }
    Game.returnToMainApplication = returnToMainApplication;
    function setGameVersionNumber(p_versionNumber) {
        callPlugin(null, null, this, exports.GAME_MODULE_NAME, "setGameVersionNumber", [p_versionNumber]);
    }
    Game.setGameVersionNumber = setGameVersionNumber;
    function setGameId(p_gameId) {
        //Set Game Id does nothing on release builds.
    }
    Game.setGameId = setGameId;
    var Leaderboard;
    (function (Leaderboard) {
        function getLeaderboardDataForLevel(p_levelNumber) {
            callPlugin(null, null, this, exports.GAME_MODULE_NAME, "getLeaderboardDataForLevel", [p_levelNumber]);
        }
        Leaderboard.getLeaderboardDataForLevel = getLeaderboardDataForLevel;
        function getLocalLeaderboardDataForLevel(p_levelNumber) {
            callPlugin(null, null, this, exports.GAME_MODULE_NAME, "getLocalLeaderboardDataForLevel", [p_levelNumber]);
        }
        Leaderboard.getLocalLeaderboardDataForLevel = getLocalLeaderboardDataForLevel;
    })(Leaderboard = Game.Leaderboard || (Game.Leaderboard = {}));
    var Mission;
    (function (Mission) {
        function saveMissionGoal(p_id, p_progress) {
            callPlugin(null, null, this, exports.GAME_MODULE_NAME, "saveMissionGoal", [p_id, p_progress]);
        }
        Mission.saveMissionGoal = saveMissionGoal;
        function saveMission(p_id, p_status, p_description) {
            callPlugin(null, null, this, exports.GAME_MODULE_NAME, "saveMission", [p_id, p_status, p_description]);
        }
        Mission.saveMission = saveMission;
        function loadMissions(p_context, p_callback) {
            callPlugin(p_callback, null, p_context, exports.GAME_MODULE_NAME, "loadMissions", []);
        }
        Mission.loadMissions = loadMissions;
        function loadStartedMissionsList(p_context, p_callback) {
            callPlugin(p_callback, null, p_context, exports.GAME_MODULE_NAME, "loadStartedMissionsList", []);
        }
        Mission.loadStartedMissionsList = loadStartedMissionsList;
    })(Mission = Game.Mission || (Game.Mission = {}));
    var SaveState;
    (function (SaveState) {
        function save(p_saveState, p_context, p_callback) {
            callPlugin(p_callback, null, p_context, exports.GAME_MODULE_NAME, "saveGameState", [p_saveState]);
        }
        SaveState.save = save;
        function load(p_context, p_callback) {
            callPlugin(p_callback, null, p_context, exports.GAME_MODULE_NAME, "loadGameState");
        }
        SaveState.load = load;
    })(SaveState = Game.SaveState || (Game.SaveState = {}));
})(Game = exports.Game || (exports.Game = {}));
var Session;
(function (Session) {
    function beginSession(p_level) {
        callPlugin(null, null, this, exports.GAME_MODULE_NAME, "beginSession", [p_level]);
    }
    Session.beginSession = beginSession;
    function endSession(p_score) {
        if (p_score === void 0) { p_score = 0; }
        callPlugin(null, null, this, exports.GAME_MODULE_NAME, "endSession", [p_score]);
    }
    Session.endSession = endSession;
})(Session = exports.Session || (exports.Session = {}));
var Goal;
(function (Goal) {
    function getShowGoalsInGame(p_context, p_callback) {
        callPlugin(p_callback, null, p_context, exports.GAME_MODULE_NAME, "getShowGoalsInGame");
    }
    Goal.getShowGoalsInGame = getShowGoalsInGame;
    function getActiveGoal(p_context, p_success, p_failure) {
        callPlugin(p_success, p_failure, p_context, exports.USER_MODULE_NAME, "getActiveGoal");
    }
    Goal.getActiveGoal = getActiveGoal;
})(Goal = exports.Goal || (exports.Goal = {}));
var Metrics;
(function (Metrics) {
    function logMetric(p_type, p_value) {
        callPlugin(null, null, this, exports.TGOMA_MODULE_NAME, "logMetric", [p_type, JSON.stringify(p_value)]);
    }
    Metrics.logMetric = logMetric;
})(Metrics = exports.Metrics || (exports.Metrics = {}));
var Misc;
(function (Misc) {
    function isConnectedToInternet(p_context, p_callback) {
        callPlugin(p_callback, null, p_context, exports.TGOMA_MODULE_NAME, "isConnectedToInternet", []);
    }
    Misc.isConnectedToInternet = isConnectedToInternet;
    function getIsConnectedToTrampoline(p_context, p_callback) {
        callPlugin(p_callback, null, p_context, exports.BLUETOOTH_MODULE_NAME, "getIsConnectedToTrampoline", []);
    }
    Misc.getIsConnectedToTrampoline = getIsConnectedToTrampoline;
    function getParentDirectory(p_context, p_callback) {
        callPlugin(p_callback, null, p_context, exports.TGOMA_MODULE_NAME, "getBaseDirectoryForGame", ["parent"]);
    }
    Misc.getParentDirectory = getParentDirectory;
})(Misc = exports.Misc || (exports.Misc = {}));
var Internal;
(function (Internal) {
    var UserManagement;
    (function (UserManagement) {
        function getFamilyAccount(p_context, p_success) {
            callPlugin(p_success, null, p_context, exports.USER_MODULE_NAME, "getFamilyAccount", []);
        }
        UserManagement.getFamilyAccount = getFamilyAccount;
        function getFamilyMember(p_context, p_success) {
            callPlugin(p_success, null, p_context, exports.USER_MODULE_NAME, "getFamilyMember", []);
        }
        UserManagement.getFamilyMember = getFamilyMember;
        function getNumFamilyMembers(p_success, p_context) {
            callPlugin(p_success, null, p_context, exports.USER_MODULE_NAME, "getNumFamilyMembers", []);
        }
        UserManagement.getNumFamilyMembers = getNumFamilyMembers;
        function getHasCompletedFtue(p_success, p_context) {
            callPlugin(p_success, null, p_context, exports.USER_MODULE_NAME, "getHasCompletedFtue", []);
        }
        UserManagement.getHasCompletedFtue = getHasCompletedFtue;
        function setHasCompletedFtue(p_hasCompletedFtue) {
            callPlugin(null, null, null, exports.USER_MODULE_NAME, "setHasCompletedFtue", [p_hasCompletedFtue]);
        }
        UserManagement.setHasCompletedFtue = setHasCompletedFtue;
        function getHasSetFamilySettings(p_success, p_context) {
            callPlugin(p_success, null, p_context, exports.USER_MODULE_NAME, "getHasSetFamilySettings", []);
        }
        UserManagement.getHasSetFamilySettings = getHasSetFamilySettings;
        function setHasSetFamilySettings(p_hasSetFamilySettings) {
            callPlugin(null, null, null, exports.USER_MODULE_NAME, "setHasSetFamilySettings", [p_hasSetFamilySettings]);
        }
        UserManagement.setHasSetFamilySettings = setHasSetFamilySettings;
        function getFamilyAccountExists(p_context, p_success) {
            callPlugin(p_success, null, p_context, exports.USER_MODULE_NAME, "getFamilyAccountExists", []);
        }
        UserManagement.getFamilyAccountExists = getFamilyAccountExists;
        function updateFamilyAccountPreferences(p_context, p_success, p_country, p_region, p_dateIsUSFormat, p_energyIsUSFormat, p_unitsAreUSFormat) {
            callPlugin(p_success, null, p_context, exports.USER_MODULE_NAME, "updateFamilyAccountPreferences", [p_country, p_region, p_dateIsUSFormat, p_energyIsUSFormat, p_unitsAreUSFormat]);
        }
        UserManagement.updateFamilyAccountPreferences = updateFamilyAccountPreferences;
    })(UserManagement = Internal.UserManagement || (Internal.UserManagement = {}));
    var Trampoline;
    (function (Trampoline) {
        function getBatteryStatus(p_success) {
            callPlugin(p_success, null, null, exports.BLUETOOTH_MODULE_NAME, "getBatteryStatus", []);
        }
        Trampoline.getBatteryStatus = getBatteryStatus;
        function getTabletLocation(p_success) {
            callPlugin(p_success, null, null, exports.TGOMA_MODULE_NAME, "getTabletLocation", []);
        }
        Trampoline.getTabletLocation = getTabletLocation;
        function getHasSetTrampolineModel(p_success, p_context) {
            callPlugin(p_success, null, p_context, exports.TGOMA_MODULE_NAME, "getHasSetTrampolineModel", []);
        }
        Trampoline.getHasSetTrampolineModel = getHasSetTrampolineModel;
        function getHasSetTrampolineOrientation(p_success, p_context) {
            callPlugin(p_success, null, p_context, exports.TGOMA_MODULE_NAME, "getHasSetTrampolineOrientation", []);
        }
        Trampoline.getHasSetTrampolineOrientation = getHasSetTrampolineOrientation;
    })(Trampoline = Internal.Trampoline || (Internal.Trampoline = {}));
    var ApplicationManagement;
    (function (ApplicationManagement) {
        function cancelDownload() {
            callPlugin(null, null, this, exports.TGOMA_MODULE_NAME, "cancelDownload", []);
        }
        ApplicationManagement.cancelDownload = cancelDownload;
        function checkForNewManifest(p_appId) {
            callPlugin(null, null, this, exports.TGOMA_MODULE_NAME, "checkForNewManifest", [p_appId.toString()]);
        }
        ApplicationManagement.checkForNewManifest = checkForNewManifest;
        function checkIfMustDownload(p_appId, p_context, p_success, p_failure) {
            callPlugin(p_success, p_failure, p_context, exports.TGOMA_MODULE_NAME, "checkIfMustDownload", [p_appId.toString()]);
        }
        ApplicationManagement.checkIfMustDownload = checkIfMustDownload;
        function checkForUpdate(p_appId, p_context, p_success, p_failure) {
            callPlugin(p_success, p_failure, p_context, exports.TGOMA_MODULE_NAME, "checkForUpdate", [p_appId.toString()]);
        }
        ApplicationManagement.checkForUpdate = checkForUpdate;
        function downloadUpdate(p_appId) {
            callPlugin(null, null, this, exports.TGOMA_MODULE_NAME, "downloadUpdate", [p_appId.toString()]);
        }
        ApplicationManagement.downloadUpdate = downloadUpdate;
        function getDownloadProgress(p_appId, p_context, p_success, p_failure) {
            callPlugin(p_success, null, p_context, exports.TGOMA_MODULE_NAME, "getDownloadProgress", [p_appId.toString()]);
        }
        ApplicationManagement.getDownloadProgress = getDownloadProgress;
        function getBaseDirectoryForGame(p_appId, p_context, p_success, p_failure) {
            callPlugin(p_success, null, p_context, exports.TGOMA_MODULE_NAME, "getBaseDirectoryForGame", [p_appId.toString()]);
        }
        ApplicationManagement.getBaseDirectoryForGame = getBaseDirectoryForGame;
        function launchApplication(p_appId, p_fullPath) {
            callPlugin(null, null, this, exports.GAME_MODULE_NAME, "launchApplication", [p_appId.toString(), p_fullPath]);
        }
        ApplicationManagement.launchApplication = launchApplication;
        function cacheParentAppLocation(p_location) {
            callPlugin(null, null, this, exports.TGOMA_MODULE_NAME, "cacheParentAppLocation", [p_location]);
        }
        ApplicationManagement.cacheParentAppLocation = cacheParentAppLocation;
        function getGoogleAnalyticsTrackingId(p_context, p_success) {
            callPlugin(p_success, null, this, exports.TGOMA_MODULE_NAME, "getGoogleAnalyticsTrackingId", []);
        }
        ApplicationManagement.getGoogleAnalyticsTrackingId = getGoogleAnalyticsTrackingId;
        function getRaygunApiKey(p_context, p_success) {
            callPlugin(p_success, null, this, exports.TGOMA_MODULE_NAME, "getRaygunApiKey", []);
        }
        ApplicationManagement.getRaygunApiKey = getRaygunApiKey;
        /**
        * Toggles the device's native wake lock functionality to stop games from sleeping.
        * As there is often no direct interaction with the tablet, some tablets will sleep during a level unless this is set.
        * @param p_wakeLock Whether the device should be locked awake. Set true for no sleeping.
        **/
        function toggleScreenTimeout(p_wakeLock) {
            callPlugin(null, null, this, exports.TGOMA_MODULE_NAME, "toggleScreenTimeout", [p_wakeLock.toString()]);
        }
        ApplicationManagement.toggleScreenTimeout = toggleScreenTimeout;
        function setParentAppVersion(p_appVersion) {
            callPlugin(null, null, this, exports.TGOMA_MODULE_NAME, "setParentAppVersion", [p_appVersion.toString()]);
        }
        ApplicationManagement.setParentAppVersion = setParentAppVersion;
        function setParentAppContent(p_appContent) {
            callPlugin(null, null, this, exports.TGOMA_MODULE_NAME, "setParentAppContent", [p_appContent.toString()]);
        }
        ApplicationManagement.setParentAppContent = setParentAppContent;
    })(ApplicationManagement = Internal.ApplicationManagement || (Internal.ApplicationManagement = {}));
})(Internal = exports.Internal || (exports.Internal = {}));
//function pollCordovaQueue() {
//    callPlugin(null, null, null, TGOMA_MODULE_NAME, "poll", [])
//}
var Enum;
(function (Enum) {
    (function (ConnectionType) {
        ConnectionType[ConnectionType["EMULATOR"] = 3] = "EMULATOR";
        ConnectionType[ConnectionType["BLUETOOTH"] = 1] = "BLUETOOTH";
        ConnectionType[ConnectionType["ANY"] = 0] = "ANY";
    })(Enum.ConnectionType || (Enum.ConnectionType = {}));
    var ConnectionType = Enum.ConnectionType;
    (function (TrampolineMessages) {
        TrampolineMessages[TrampolineMessages["ID_EVENT_IMPACT"] = 192] = "ID_EVENT_IMPACT";
        TrampolineMessages[TrampolineMessages["ID_EVENT_DEPART"] = 194] = "ID_EVENT_DEPART";
        TrampolineMessages[TrampolineMessages["ID_EVENT_13ZLOC"] = 196] = "ID_EVENT_13ZLOC";
        TrampolineMessages[TrampolineMessages["ID_EVENT_5ZLOC"] = 198] = "ID_EVENT_5ZLOC";
        TrampolineMessages[TrampolineMessages["ID_EVENT_HEIGHT"] = 204] = "ID_EVENT_HEIGHT";
        TrampolineMessages[TrampolineMessages["ID_EVENT_AIRTIME"] = 206] = "ID_EVENT_AIRTIME";
        TrampolineMessages[TrampolineMessages["ID_EVENT_MATTIME"] = 208] = "ID_EVENT_MATTIME";
        TrampolineMessages[TrampolineMessages["ID_EVENT_BOUNCENUM"] = 210] = "ID_EVENT_BOUNCENUM";
        TrampolineMessages[TrampolineMessages["ID_EVENT_CONSECBOUNCE"] = 212] = "ID_EVENT_CONSECBOUNCE";
        TrampolineMessages[TrampolineMessages["ID_EVENT_ACTIVITY"] = 214] = "ID_EVENT_ACTIVITY";
        TrampolineMessages[TrampolineMessages["ID_EVENT_PASSIVITY"] = 216] = "ID_EVENT_PASSIVITY";
        TrampolineMessages[TrampolineMessages["ID_EVENT_CALORIES"] = 218] = "ID_EVENT_CALORIES";
        TrampolineMessages[TrampolineMessages["ID_EVENT_BATTLOW"] = 220] = "ID_EVENT_BATTLOW";
        TrampolineMessages[TrampolineMessages["ID_EVENT_POLAR_LOC"] = 222] = "ID_EVENT_POLAR_LOC";
    })(Enum.TrampolineMessages || (Enum.TrampolineMessages = {}));
    var TrampolineMessages = Enum.TrampolineMessages;
})(Enum = exports.Enum || (exports.Enum = {}));

});
