var manifestDoc;
var fileSystem;
var updateRef;
var progressRef;
var skipTimeout;

var DISCONNECTED = 0;
var WIFI = 1;
var MOBILE_DATA = 2;

var require,
    define;

document.addEventListener("deviceready", onDeviceReady, false);
console.log = function () { };
function onDeviceReady() {
    console.log("onDeviceReady()");
    if (screen.lockOrientation != undefined) {
        // Deprecated
        screen.lockOrientation('landscape');
    }
    else {
        screen.orientation.lock('landscape');
    }
    Tgoma.LanguageManager.initialise("", function () {
        $(".progressbar-text").text(Tgoma.LanguageManager.getString("loading"));

        AndroidFullScreen.immersiveMode();
        $("#progressbar").progressbar({
            value: false
        });

        $("#skipButton").click(onSkipButtonClicked);
        $("#skipButton").hide();

        if (window.navigator.network.connection.type == "none") {
            $(".progressbar-text").text(Tgoma.LanguageManager.getString("waiting_for_network"));
        }

        parent.document.addEventListener("offline", onNetworkConnectionLost, false);

        setTimeout(initNative, 1000);
    });
}

function initNative() {
    console.log("initNative");

    DialogPlugin.initialise();
    parent.TgomaMessaging.callPlugin(null, null, this, "SpringfreeMessaging", "initialise", []);

    var deviceLanguage = navigator.language;
    var languageTag = deviceLanguage.substr(0, 2);
    parent.TgomaMessaging.callPlugin(null, null, this, parent.TgomaMessaging.TGOMA_MODULE_NAME, "setLanguage", [languageTag]);

    setTimeout(initStorage, 1500);
}

function initStorage() {
    console.log("initStorage");

    if (Tgoma.ApplicationUtils.DeviceUtils.isMobile.Android()) {
        //AndroidFullScreen.resetWindow();
        onCordovaSetupComplete();
        //AndroidFullScreen.immersiveMode(settingLeanModeSuccess, settingLeanModeFailed);
    } else if (Tgoma.ApplicationUtils.DeviceUtils.isMobile.iOS()) {
        window.resolveLocalFileSystemURL(cordova.file.dataDirectory, function (entry) {
            entry.getDirectory("www", { create: true }, checkForChanges);
        });
    }
}

///////////////////////////////////////////////
// File Copying
///////////////////////////////////////////////

function checkForChanges() {
    var oldFile = cordova.file.dataDirectory + "www/plugins/pluginConfig.json";
    var newFile = cordova.file.applicationDirectory + "www/plugins/pluginConfig.json";

    window.resolveLocalFileSystemURL(oldFile,
            function (fileEntry1) {
                window.resolveLocalFileSystemURL(newFile,
                    function (fileEntry2) {
                        var reader1 = new FileReader();
                        var version1;

                        reader1.onloadend = function (evt) {
                            var reader2 = new FileReader();
                            var version2;

                            version1 = JSON.parse(evt.target.result).version;
                            reader2.onloadend = function (evt) {
                                version2 = JSON.parse(evt.target.result).version;
                                copyFiles(version1 != version2);
                            }
                            reader2.readAsText(fileEntry2);
                        }
                        reader1.readAsText(fileEntry1);
                    },
                    errorCallback);
            },
            function () {
                // If we can't find a file in the destination then either we are doing a fresh
                // install or upating from an old install prior to this change.
                copyFiles(true);
            });
}

function copyFiles(hasChanged) {
    // Copy cordova, followed by cordova_plugins, then the plugins/ folder.
    copyFile(cordova.file.applicationDirectory + "www/", cordova.file.dataDirectory + "www/", "cordova.js", false, function () {
        copyFile(cordova.file.applicationDirectory + "www/", cordova.file.dataDirectory + "www/", "cordova_plugins.js", hasChanged, function () {
            copyDirectory(cordova.file.applicationDirectory + "www/", cordova.file.dataDirectory + "www/", "plugins", hasChanged, onCordovaSetupComplete);
        });
    });
}

function copyFile(p_srcDir, p_destDir, filename, overwrite, successCallback) {
    console.log("copyFile(" + filename + ")")

    if (!overwrite) {
        // Check to see if we have already copied the file...
        window.resolveLocalFileSystemURL(p_destDir + filename, successCallback, function () {
            // We haven't
            doFileCopy(p_srcDir, filename, p_destDir, successCallback);
        });
    }
    else {
        // Check to see if we have already copied the file...
        window.resolveLocalFileSystemURL(p_destDir + filename, function (dstEntry) {
            // If we have, delete it and then copy
            dstEntry.remove(function () {
                doFileCopy(p_srcDir, filename, p_destDir, successCallback);
            }, errorCallback, errorCallback);
        }, function () {
            // If we haven't just copy
            doFileCopy(p_srcDir, filename, p_destDir, successCallback);
        });
    }
}

function doFileCopy(p_srcDir, filename, p_destDir, successCallback) {
    // If we haven't, try to find the src file
    window.resolveLocalFileSystemURL(p_srcDir + filename,
    function (fileEntry) {
        console.log("copyFile() LocalFileSystemURL resolved src")
        // We got the src - now get a DirectoryEntry for the dest
        window.resolveLocalFileSystemURL(p_destDir,
            function (dirEntry) {
                console.log("copyFile() LocalFileSystemURL resolved dest")
                // move the file to a new directory and rename it
                fileEntry.copyTo(dirEntry, null, successCallback, errorCallback);
            },
            errorCallback);
    },
    errorCallback);
}

function copyDirectory(p_srcDir, p_destDir, directory, overwrite, successCallback) {
    console.log("copyDirectory(" + directory + ")")

    if (!overwrite) {    // Check to see if we have the directory first 
        resolveLocalFileSystemURL(p_destDir + directory,
            // If we do, we'll hit the successCallback
            successCallback,
            // If we don't, we'll hit the error callback (and we'll copy the dir)
            function () {
                doDirectoryCopy(p_srcDir, directory, p_destDir, successCallback);
            });
    }
    else {
        resolveLocalFileSystemURL(p_destDir + directory,
            // If we do, we'll delete it before copying
            function (dirEntry) {
                dirEntry.removeRecursively(function () {
                    doDirectoryCopy(p_srcDir, directory, p_destDir, successCallback);
                }, errorCallback, errorCallback);                
            },
            // If we don't, we'll hit the error callback (and we'll copy the dir)
            function () {
                doDirectoryCopy(p_srcDir, directory, p_destDir, successCallback);
            });
    }
}

function doDirectoryCopy(p_srcDir, directory, p_destDir, successCallback) {
    // Find the srcDir
    resolveLocalFileSystemURL(p_srcDir + directory, function (srcDir) {
        // Find the destDir
        resolveLocalFileSystemURL(p_destDir, function (destDir) {
            // copy the srcDir to the destDir
            srcDir.copyTo(destDir, null, successCallback, errorCallback);
        }, errorCallback)
    }
    , errorCallback);
}
///////////////////////////////////////////////

function onCordovaSetupComplete() {
    console.log("onCordovaSetupComplete()");
    skipTimeout = setTimeout(allowSkip, 30000); // Allow a max of 30 seconds no matter what is happening before giving the option to skip download.

    parent.document.addEventListener("online", onNetworkConnectionGained, false);
    parent.document.addEventListener("onCheckForNewManifestReturned", onCheckForNewManifestReturned, false);
    parent.document.addEventListener("onDownloadStarted", onDownloadStarted, false);
    parent.document.addEventListener("onDownloadProgress", onDownloadProgress, false);
    //parent.document.addEventListener("onDownloadComplete", onDownloadComplete, false);

    // Time to update the parent app (silently).
    if (window.navigator.network.connection.type == "none") {
        TgomaMessaging.Internal.UserManagement.getFamilyAccountExists(this, onFamilyExistsReturned);
    } else {
        Tgoma.DownloadUtils.onGameClicked("parent", onDownloadProgress, true, null, onDownloadComplete);
        Tgoma.DownloadUtils.m_manifestReturnedRef = setTimeout(function () {
            //onDownloadComplete(false); //Uncomment this line so if it can't get a manifest for some reason it'll show the skip button.
            TgomaMessaging.Internal.UserManagement.getFamilyAccountExists(this, onFamilyExistsReturned); //Uncomment this line to have the app skip automatically instead of showing the skip button.
        }, 15000);
    }
}

function onNetworkConnectionLost() {
    console.log("onNetworkConnectionLost()");
    $(".progressbar-text").text(Tgoma.LanguageManager.getString("waiting_for_network"));
    skipTimeout = setTimeout(allowSkip, 5000);
}

function onNetworkConnectionGained() {
    console.log("onNetworkConnectionGained()");
    $(".progressbar-text").text(Tgoma.LanguageManager.getString("resuming_download"));

    // clearTimeout(skipTimeout);
    // $("#skipButton").hide();

    // Retry the download
    Tgoma.DownloadUtils.onGameClicked("parent", onDownloadProgress, true, null, onDownloadComplete);
}

// This really should take into account whether the update is mandatory 
// and whether they have a local copy to skip to
function allowSkip() {
    console.log("allowSkip()");
    $("#skipButton").css("visibility", "");
    $("#skipButton").show();
}

function onSkipButtonClicked() {
    console.log("onSkipButtonClicked()");
    $("#skipButton").unbind();

    // Stop the download.
    TgomaMessaging.callPlugin(null, null, this, TgomaMessaging.TGOMA_MODULE_NAME, "cancelDownload", []);
    TgomaMessaging.Internal.UserManagement.getFamilyAccountExists(this, onFamilyExistsReturned);
}

function onCheckForNewManifestReturned(e) {
    console.log("onCheckForNewManifestReturned()");

    if (e.p_hasUpdate == true) {
        $(".progressbar-text").text(Tgoma.LanguageManager.getString("update_found"));
    }

    parent.document.removeEventListener("onCheckForNewManifestReturned", onCheckForNewManifestReturned);
}

// This is called by the DownloadUtils (from the C++)
function onDownloadProgress(e) {
    var _percent = e["progress"]
    console.log("onGetDownloadProgressSucceeded(" + _percent + ")")
    if (!isNaN(_percent)) {
        $("#progressbar").progressbar("option", {
            value: _percent * 100
        });
    }
}

function onDownloadStarted(e) {
    console.log("onDownloadStarted()");
    $(".progressbar-text").text(Tgoma.LanguageManager.getString("downloading"));
}

// This is called by the DownloadUtils (from the C++)
function onDownloadComplete(p_successful) {
    console.log("onDownloadComplete(" + p_successful + ")");

    // If the event indicates a successful download
    if (p_successful) {

        // Remove the online/offline listeners
        parent.document.removeEventListener("offline", onNetworkConnectionLost);
        parent.document.removeEventListener("online", onNetworkConnectionGained);

        // Hide the skip button or it might cause some issues
        $("#skipButton").hide();

        // Change the text
        //$(".progressbar-text").text("Download complete, installing...");

        // Force the progress bar to be full as the CC tools don't account for 
        // the largest asset's filesize, resulting in a bar at ~80% full
        $("#progressbar").progressbar("option", {
            value: 100
        });

        TgomaMessaging.Internal.UserManagement.getFamilyAccountExists(this, onFamilyExistsReturned);
    } else {
        // We've identified an issue - the actual checking will be occurring inside DownloadUtils.
        $(".progressbar-text").text(Tgoma.LanguageManager.getString("checking_connectivity"));
        allowSkip();
    }
}

function errorCallback() {
    console.log(arguments);
}

function onFamilyExistsReturned(p_familyAccountExists) {
    console.log("onFamilyExistsReturned(" + p_familyAccountExists + ")");

    var _path
    if (p_familyAccountExists) {
        _path = "default.htm";
    } else {
        _path = "familyLogin.htm";
    }

    TgomaMessaging.callPlugin(null, null, this, TgomaMessaging.TGOMA_MODULE_NAME, "moveParentApp", []); //Move the parent app if it hasn't been downloaded yet.
    Tgoma.DownloadUtils.locateGame("parent", _path);
}

function settingLeanModeSuccess() {
    console.log("SettingLeanModeSuccess()")
}

function settingLeanModeFailed() {
    console.log("SettingLeanModeFailed()")
}